/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program will generate a key agreed upon value using a previously-
    generated DH parameters.  You would not want the DH parameters hard-coded
    in a real application.
 */

#include <stdio.h>   
#include <string.h>  
#include <time.h>
#include "aglobal.h"
#include "bsafe.h"

B_ALGORITHM_METHOD *DH_AGREE_SAMPLE_CHOOSER[] = {
  &AM_SHA_RANDOM,
  &AM_DH_KEY_AGREE,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

int GeneralSurrenderFunction PROTO_LIST ((POINTER handle));

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  ITEM dhParametersBER;
  
  static unsigned char dhParamsBER[154] = {
      0x30, 0x81, 0x97, 0x06, 0x09, 0x2a, 0x86, 0x48,
      0x86, 0xf7, 0x0d, 0x01, 0x03, 0x01, 0x30, 0x81,
      0x89, 0x02, 0x41, 0x00, 0xaf, 0xbb, 0x7e, 0xe7,
      0xf3, 0xc2, 0x6c, 0xa0, 0x4e, 0xab, 0x0d, 0x62,
      0xea, 0x0d, 0x3f, 0xdd, 0xce, 0x97, 0x8b, 0xe1,
      0xe8, 0x6b, 0xbe, 0x67, 0x21, 0x1c, 0x59, 0x5c,
      0x15, 0x41, 0x0e, 0x5b, 0xe4, 0xe5, 0x47, 0x39,
      0xc6, 0x0d, 0x3f, 0xc3, 0x22, 0xb9, 0x64, 0x58,
      0xc8, 0x40, 0x4c, 0x0e, 0x61, 0xea, 0xb7, 0x33,
      0xfa, 0xfa, 0x2a, 0xa5, 0x4c, 0x6a, 0x06, 0x48,
      0x60, 0xa8, 0x7f, 0xc1, 0x02, 0x40, 0x68, 0x66, 
      0x62, 0x61, 0x34, 0x6b, 0x19, 0x5e, 0xd6, 0x73,
      0x38, 0x6a, 0x94, 0x21, 0xeb, 0xd0, 0x61, 0x4f,
      0x20, 0xc6, 0x33, 0xc1, 0x69, 0x08, 0x67, 0xd0,
      0x40, 0x03, 0xac, 0x00, 0x69, 0x5e, 0x2e, 0x34,
      0x0e, 0xca, 0x38, 0xb2, 0x1b, 0x18, 0x65, 0x52,
      0x32, 0x16, 0x08, 0x9a, 0x4f, 0x68, 0x8b, 0x01,
      0x4f, 0x0a, 0xb3, 0x6a, 0xfd, 0xa7, 0x16, 0xad,
      0xe7, 0x90, 0x4c, 0x8a, 0x73, 0xbf, 0x02, 0x02,
      0x01, 0xf8 
  };

  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ dhKeyAgreeAlg = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ otherPartyAlg = (B_ALGORITHM_OBJ)NULL_PTR;
  
  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;
  
  A_SURRENDER_CTX generalSurrenderContext;
  int generalFlag;

  A_DH_KEY_AGREE_PARAMS *getParams;
    
  unsigned char *myPublicValue = NULL_PTR;
  unsigned int myPublicValueLen;

  unsigned char *otherPublicValue = NULL_PTR;
  unsigned int otherPublicValueLen;

  unsigned char *agreedUponSecretValue = NULL_PTR;
  unsigned int agreedUponSecretValueLen;

  unsigned char *otherAgreedUponValue = NULL_PTR;
  unsigned int otherAgreedUponValueLen;
        
  unsigned int status;

  generalSurrenderContext.Surrender = GeneralSurrenderFunction;
  generalSurrenderContext.handle = (POINTER)&generalFlag;
  generalSurrenderContext.reserved = NULL_PTR;

  do {
    printf ("Diffie-Hellman Key Agreement Algorithm \n");
    printf ("====================================== \n");

    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */
    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, DH_AGREE_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
         break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed for private value");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    /*  ========================================================  */

    printf ("\n   Use previously-generated DH parameters\n");
    printf ("   ====================================== \n");
    
    dhParametersBER.len = 154;
    dhParametersBER.data = (unsigned char *)dhParamsBER;

    printf ("\n   Key Agreement -- Phase 1 \n");
    printf ("   ======================== \n");

    /*  Step 1:  Create algorithm object
     */
    if ((status = B_CreateAlgorithmObject (&dhKeyAgreeAlg)) != 0)
      break;

    if ((status = B_CreateAlgorithmObject (&otherPartyAlg)) != 0)
      break;

    /*  Step 2:  Set the algorithm object to AI_DHKeyAgreeBER
     */
    if ((status = B_SetAlgorithmInfo
         (dhKeyAgreeAlg, AI_DHKeyAgreeBER,
          (POINTER)&dhParametersBER)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (otherPartyAlg, AI_DHKeyAgreeBER,
          (POINTER)&dhParametersBER)) != 0)
      break;

    /*  Step 3:  Init
     */
    if ((status = B_KeyAgreeInit
         (dhKeyAgreeAlg, (B_KEY_OBJ)NULL_PTR, DH_AGREE_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    if ((status = B_KeyAgreeInit
         (otherPartyAlg, (B_KEY_OBJ)NULL_PTR, DH_AGREE_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Phase 1
     */
    /*  Find out how big the prime is so we know how many bytes to
        allocate for the public value buffer.
     */
    if ((status = B_GetAlgorithmInfo
         ((POINTER *)&getParams, dhKeyAgreeAlg, AI_DHKeyAgree)) != 0)
      break;
    
    myPublicValue = T_malloc (getParams->prime.len);
    if ((status = (myPublicValue == NULL_PTR)) != 0)
      break;
    
    otherPublicValue = T_malloc (getParams->prime.len);
    if ((status = (otherPublicValue == NULL_PTR)) != 0)
      break;

    /*  generalFlag is for the surrender function */
    generalFlag = 0;
    if ((status = B_KeyAgreePhase1
         (dhKeyAgreeAlg, myPublicValue, &myPublicValueLen,
          getParams->prime.len, randomAlgorithm,
          &generalSurrenderContext)) != 0)
      break;
    printf ("\n  The public value to give to the other party");
    printf (" (%u bytes):\n", myPublicValueLen);
    PrintBuf (myPublicValue, myPublicValueLen);

    generalFlag = 0;
    if ((status = B_KeyAgreePhase1
         (otherPartyAlg, otherPublicValue, &otherPublicValueLen,
          getParams->prime.len, randomAlgorithm,
          &generalSurrenderContext)) != 0)
      break;
    printf ("\n  The public value received from the other party");
    printf (" (%u bytes):\n", otherPublicValueLen);
    PrintBuf (otherPublicValue, otherPublicValueLen);

    printf ("\n   Key Agreement -- Phase 2 \n");
    printf ("   ======================== \n");

    /*  Step 5:  Phase 2
                 The other party should send their public value and its
                 length.    
     */
    agreedUponSecretValue = T_malloc (getParams->prime.len);
    if ((status = (agreedUponSecretValue == NULL_PTR)) != 0)
      break;

    otherAgreedUponValue = T_malloc (getParams->prime.len);
    if ((status = (otherAgreedUponValue == NULL_PTR)) != 0)
      break;

    generalFlag = 0;
    if ((status = B_KeyAgreePhase2
         (dhKeyAgreeAlg, agreedUponSecretValue,
          &agreedUponSecretValueLen, getParams->prime.len,
          otherPublicValue, otherPublicValueLen,
          &generalSurrenderContext)) != 0)
      break;
    printf ("\n  The value derived by me");
    printf (" (%u bytes):\n", agreedUponSecretValueLen);
    PrintBuf (agreedUponSecretValue, agreedUponSecretValueLen);

    generalFlag = 0;
    if ((status = B_KeyAgreePhase2
         (otherPartyAlg, otherAgreedUponValue,
          &otherAgreedUponValueLen, getParams->prime.len,
          myPublicValue, myPublicValueLen,
          &generalSurrenderContext)) != 0)
      break;
    printf ("\n  The value derived by the other party");
    printf (" (%u bytes):\n", otherAgreedUponValueLen);
    PrintBuf (otherAgreedUponValue, otherAgreedUponValueLen);

    if ((status = (agreedUponSecretValueLen != otherAgreedUponValueLen)) != 0)
      break;

    if ((status = T_memcmp (agreedUponSecretValue, otherAgreedUponValue,
                            agreedUponSecretValueLen)) != 0)
      break;

    printf ("\nKeys agree\n");

  } while (0);
    
  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Key Agreement failed");
  }  
  
  /*  Step 6:  Destroy
   */
  B_DestroyAlgorithmObject (&dhKeyAgreeAlg);
  B_DestroyAlgorithmObject (&otherPartyAlg);
  B_DestroyAlgorithmObject (&randomAlgorithm);

  /*  Free up any memory allocated
   */
  T_free (randomSeed);
  T_free (myPublicValue);
  T_free (agreedUponSecretValue);
  T_free (otherPublicValue);
  T_free (otherAgreedUponValue);

} /*  end main  */

/*   General Surrender Function   */
/*   ==========================   */
int GeneralSurrenderFunction (handle)
POINTER handle;
{
  char s[100];
  static time_t currentTime;
  time_t getTime;

  if ((int)*handle == 0) {
    getTime = time(NULL);
    strftime (s, 100, "%H:%M:%S on %A, %d %B %Y", localtime(&getTime));
    printf ("\n%s\n", s);
    printf ("Surrender function ...\n");
    *handle = 1;
    time (&currentTime);
  }
  else {
    time (&getTime);
    if (currentTime != getTime) {
      printf (" .");
      currentTime = getTime;
    }
  }
  return (0);
} /*  end GeneralSurrenderFunction  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
