/* Copyright (C) RSA Data Security, Inc. created 1995, modified 1997.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "aglobal.h"
#include "bsafe.h"

#define KEY_SIZE 24

B_ALGORITHM_METHOD *HMAC_CHOOSER[] = {
  &AM_SHA,
  &AM_SHA_RANDOM,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

int InitializeRandomAlgorithm PROTO_LIST ((B_ALGORITHM_OBJ *));
void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main() {

  B_ALGORITHM_OBJ HMACDigesterA = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ HMACDigesterB = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;

  B_KEY_OBJ HMACKeyA = (B_KEY_OBJ)NULL_PTR;
  B_KEY_OBJ HMACKeyB = (B_KEY_OBJ)NULL_PTR;

  unsigned char *keyData;

  unsigned char dataToDigest[] = "Digest this sentence.";
  unsigned int dataToDigestLen = strlen ((char *)dataToDigest);
  unsigned char *digestedData;
  unsigned int digestedDataLen;

  B_DIGEST_SPECIFIER hmacInfo;

  unsigned int status;

  do {

    printf ("HMAC Authenticated Message Example\n");
    printf ("==================================\n");
    printf ("Hash algorithm = SHA1\n");

    if ((status = InitializeRandomAlgorithm (&randomAlgorithm)) != 0)
      break;

    /* Create a key object */
    if ((status = B_CreateKeyObject (&HMACKeyA)) != 0)
      break;

    keyData = T_malloc (KEY_SIZE);
    if ((status = (keyData == NULL_PTR)) != 0)
      break;

    /* Generate KEY_SIZE bytes of random data for the key */
    if ((status = B_GenerateRandomBytes
	 (randomAlgorithm, keyData, KEY_SIZE,
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /* Set the key object */
    if ((status = B_SetKeyInfo (HMACKeyA, KI_24Byte, keyData)) != 0)
      break;

    printf ("\nRandom Bytes supplied for key:  \n");
    PrintBuf (keyData, 24);
    
    /* Zeroize and free the key data memory */
    T_memset (keyData, 0, KEY_SIZE);
    T_free (keyData);
    keyData = NULL_PTR;

    printf ("\nData to be digested: \n");
    PrintBuf (dataToDigest, dataToDigestLen);

    /* Step 1:  Create the HMAC digesting algorithm object */
    if ((status = B_CreateAlgorithmObject (&HMACDigesterA)) != 0)
      break;

    /* Step 2: Set the algorithm object */
    /* Set for HMAC using SHA1 */
    hmacInfo.digestInfoType = AI_SHA1;
    hmacInfo.digestInfoParams = NULL_PTR;

    if ((status = B_SetAlgorithmInfo 
	 (HMACDigesterA, AI_HMAC, (POINTER)&hmacInfo)) != 0)
      break;

    /* Step 3:  Initalize the algorithm object for digesting */
    if ((status = B_DigestInit 
	 (HMACDigesterA, HMACKeyA, HMAC_CHOOSER, 
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /* Step 4:  Update */
    if ((status = B_DigestUpdate
	 (HMACDigesterA, dataToDigest, dataToDigestLen,
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /* Step 5:  Finalize the process */
    /* Allocate space for the digested data */
    digestedData = T_malloc (20);
    if ((status = (digestedData == NULL_PTR)) != 0)
      break;

    if ((status = B_DigestFinal
	 (HMACDigesterA, digestedData, &digestedDataLen,
	  20, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    
    /* now print out the digest */
    printf ("\nDigested data: \n");
    PrintBuf (digestedData, digestedDataLen);

    /* zeroize and free the digest */
    T_memset (digestedData, 0, 20);
    T_free (digestedData);
    digestedData = NULL_PTR;
    digestedDataLen = 0;

    /* ============================================== */
    /* now hash again to demonstrate with another key */

    /* Create a key object */
    if ((status = B_CreateKeyObject (&HMACKeyB)) != 0)
      break;

    keyData = T_malloc (KEY_SIZE);
    if ((status = (keyData == NULL_PTR)) != 0)
      break;

    /* Generate KEY_SIZE bytes of random data for the key */
    if ((status = B_GenerateRandomBytes
	 (randomAlgorithm, keyData, KEY_SIZE,
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /* Set the key object */
    if ((status = B_SetKeyInfo (HMACKeyB, KI_24Byte, keyData)) != 0)
      break;

    printf ("\nRandom Bytes supplied for key:  \n");
    PrintBuf (keyData, 24);
    
    /* Zeroize and free the key data memory */
    T_memset (keyData, 0, KEY_SIZE);
    T_free (keyData);
    keyData = NULL_PTR;

    printf ("\nData to be digested: \n");
    PrintBuf (dataToDigest, dataToDigestLen);

    /* Step 1:  Create the HMAC digesting algorithm object */
    if ((status = B_CreateAlgorithmObject (&HMACDigesterB)) != 0)
      break;

    /* Step 2: Set the algorithm object */
    if ((status = B_SetAlgorithmInfo 
	 (HMACDigesterB, AI_HMAC, (POINTER)&hmacInfo)) != 0)
      break;

    /* Step 3:  Initalize the algorithm object for digesting */
    if ((status = B_DigestInit 
	 (HMACDigesterB, HMACKeyB, HMAC_CHOOSER, 
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /* Step 4:  Update */
    if ((status = B_DigestUpdate
	 (HMACDigesterB, dataToDigest, dataToDigestLen,
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /* Step 5:  Finalize the process */
    /* Allocate space for the digested data */
    digestedData = T_malloc (20);
    if ((status = (digestedData == NULL_PTR)) != 0)
      break;

    if ((status = B_DigestFinal
	 (HMACDigesterB, digestedData, &digestedDataLen,
	  20, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    
    /* now print out the digest */
    printf ("\nDigested data: \n");
    PrintBuf (digestedData, digestedDataLen);
    
    /* zeroize and free the digest */
    T_memset (digestedData, 0, 20);
    T_free (digestedData);
    digestedData = NULL_PTR;
    digestedDataLen = 0;

  } while (0);
  
  /* Step 6:  Destroy */
  B_DestroyKeyObject (&HMACKeyA);
  B_DestroyKeyObject (&HMACKeyB);
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&HMACDigesterA);
  B_DestroyAlgorithmObject (&HMACDigesterB);
  
  if (digestedData != NULL_PTR) {
    T_memset (digestedData, 0, 20);
    T_free (digestedData);
    digestedData = NULL_PTR;
    digestedDataLen = 0;
  }

  if (keyData != NULL_PTR) {
    T_memset (keyData, 0, 20);
    T_free (keyData);
    keyData = NULL_PTR;
  }

  printf ("Exited with status = %d\n", status);
}

/*  Create a random algorithm
 *  Pass in a pointer to a B_ALGORITHM_OBJ and be sure to call
 *  B_DestroyAlgorithmObject later in the function that calls 
 *  this one.
 */
int InitializeRandomAlgorithm (randomAlgorithm)
B_ALGORITHM_OBJ *randomAlgorithm;
{
  
  B_ALGORITHM_METHOD *RAND_CHOOSER[] = {
    &AM_SHA_RANDOM,
    (B_ALGORITHM_METHOD *)NULL_PTR
  };

  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;

  unsigned int status;

  do {
    if ((status = B_CreateAlgorithmObject (randomAlgorithm)) != 0)
      break;
    
    if ((status = B_SetAlgorithmInfo 
	 (*randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit 
	 (*randomAlgorithm, RAND_CHOOSER, 
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 80;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
      break;

    T_memset (randomSeed, 0, randomSeedLen);
    
    puts ("Enter a random seed");
    if ((status = 
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;
    
    if ((status = B_RandomUpdate
	 (*randomAlgorithm, randomSeed, randomSeedLen, 
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

  } while (0);

  T_free (randomSeed);
  
  if(status != 0)
     printf ("Failed to initialize random algorithm.  Status = %i\n", status);
      
  return (status);

}  /* end InitializeRandomAlgorithm */

/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
}
