/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program encrypts & decrypts using RC2 in CBC mode.
 */

#include <stdio.h>      
#include <string.h>     
#include "aglobal.h"
#include "bsafe.h"

#define BLOCK_SIZE 8
#define KEY_SIZE 24

B_ALGORITHM_METHOD *RC2_CHOOSER[] = {
  &AM_CBC_ENCRYPT,
  &AM_CBC_DECRYPT,
  &AM_RC2_ENCRYPT,
  &AM_RC2_DECRYPT,
  &AM_SHA_RANDOM,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_KEY_OBJ rc2Key = (B_KEY_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ rc2Encrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ rc2Decrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;

  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;
  
  unsigned char initVector[BLOCK_SIZE];
  ITEM ivItem;
  ITEM rc2KeyItem;
  A_RC2_PARAMS rc2Params;

  B_BLK_CIPHER_W_FEEDBACK_PARAMS fbParams;

  unsigned char *dataToEncrypt = (unsigned char *)"Encrypt this sentence.";
  unsigned int dataToEncryptLen;

  unsigned char *encryptedData = NULL_PTR;
  unsigned int outputLenUpdate, outputLenFinal, outputLenTotal;
  unsigned int encryptedDataLen;

  unsigned char *decryptedData = NULL_PTR;
  unsigned int decryptedLenUpdate, decryptedLenFinal;
  unsigned int decryptedLenTotal, decryptedDataLen;
  
  unsigned int status;

  rc2KeyItem.data = NULL_PTR;

  do {
    printf ("RC2 w/CBC Pad algorithm:  Encryption phase \n");
    printf ("========================================== \n");

    printf ("Input data: %s \n", dataToEncrypt);
    dataToEncryptLen = strlen ((char *)dataToEncrypt) + 1;
    printf ("Input data in hex (%u bytes):\n", dataToEncryptLen);
    PrintBuf ((unsigned char *)dataToEncrypt, dataToEncryptLen);

    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */
    printf ("Generating random bytes \n");
    printf ("======================= \n");

    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, RC2_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
      break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  ========================================================  */
    
    /*  B_GenerateRandomBytes is called here for 8 bytes, the block size.
     */  
    if ((status = B_GenerateRandomBytes
         (randomAlgorithm, (unsigned char *)initVector, BLOCK_SIZE,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    
    printf ("%i bytes of random iv: \n", BLOCK_SIZE);
    PrintBuf (initVector, BLOCK_SIZE);

    /*  Step 1:  Create an algorithm object. */
    if ((status = B_CreateAlgorithmObject (&rc2Encrypter)) != 0)
      break;

    /*  Step 2:  Set the algorithm to a type that does RC2 encryption.
          AI_RC2_CBCPad will do. This algorithm type needs an 8-byte
          initialization vector (IV), and an effective key size in bits.
          Both are set by declaring a struct of type, A_RC2_CBC_PARAMS.
          The IV will be set with 8 random bytes, and the effective key
          bits will be set to 80.
     */
    rc2Params.effectiveKeyBits = 80;

    ivItem.data = initVector;
    ivItem.len = BLOCK_SIZE;
    
    fbParams.encryptionMethodName = (unsigned char *)"rc2";
    fbParams.encryptionParams = (POINTER)&rc2Params;
    fbParams.feedbackMethodName = (unsigned char *)"cbc";
    fbParams.feedbackParams = (POINTER)&ivItem;
    fbParams.paddingMethodName = (unsigned char *)"pad";
    fbParams.paddingParams = NULL_PTR;
    
    if ((status = B_SetAlgorithmInfo
         (rc2Encrypter, AI_FeedbackCipher, (POINTER)&fbParams)) != 0)
      break;

       /*  Step 3a:  Create a key object. */
       if ((status = B_CreateKeyObject (&rc2Key)) != 0)
         break;

       /*  Step 3b:  Set the key object with a random 24-byte key
        */
       rc2KeyItem.len = KEY_SIZE;
       rc2KeyItem.data = T_malloc (rc2KeyItem.len);
       if ((status = (rc2KeyItem.data == NULL_PTR)) != 0)
         break;

       if ((status = B_GenerateRandomBytes
            (randomAlgorithm, rc2KeyItem.data, rc2KeyItem.len,
             (A_SURRENDER_CTX *)NULL_PTR)) != 0)
         break;  
    
       printf ("%i-byte random RC2 key: \n", rc2KeyItem.len);
       PrintBuf (rc2KeyItem.data, rc2KeyItem.len);
       
       if ((status = B_SetKeyInfo
            (rc2Key, KI_Item, (POINTER)&rc2KeyItem)) != 0)
         break;

       /*  Let's zeroize the memory and free it up immediately after
             setting the key for security reasons.
        */
       if (rc2KeyItem.data != NULL_PTR) {
         T_memset (rc2KeyItem.data, 0, rc2KeyItem.len);
         T_free (rc2KeyItem.data);
         rc2KeyItem.data = NULL_PTR;
         rc2KeyItem.len = 0;
       }
  
    /*  Step 3:  Init */
    if ((status = B_EncryptInit
         (rc2Encrypter, rc2Key, RC2_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update
     */
    encryptedDataLen = dataToEncryptLen + BLOCK_SIZE;
    encryptedData = T_malloc (encryptedDataLen);
    if ((status = (encryptedData == NULL_PTR)) != 0)
      break;

    if ((status = B_EncryptUpdate
         (rc2Encrypter, encryptedData, &outputLenUpdate,
          encryptedDataLen, dataToEncrypt, dataToEncryptLen,
	  (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 5:  Final */
    if ((status = B_EncryptFinal
         (rc2Encrypter, encryptedData + outputLenUpdate,
          &outputLenFinal, encryptedDataLen - outputLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("Encrypted data: \n");
    outputLenTotal = outputLenUpdate + outputLenFinal;
    PrintBuf (encryptedData, encryptedDataLen);
    printf ("\n");

    printf ("RC2 w/CBC Pad algorithm:  Decryption phase \n");
    printf ("========================================== \n");

    if ((status = B_CreateAlgorithmObject (&rc2Decrypter)) != 0)
      break;

    /*  Use the same AI and initialization vector as for encryption
     */
    if ((status = B_SetAlgorithmInfo
         (rc2Decrypter, AI_FeedbackCipher, (POINTER)&fbParams)) != 0)
      break;

    /*  Use the same key as for encryption
     */
    if ((status = B_DecryptInit
         (rc2Decrypter, rc2Key, RC2_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /* Set the buffer that will take the decrypted data to be the same size
     * as the encrypted data's buffer.  With AI_RC2_CBCPad, there will be
     * fewer bytes in the decrypted data than in the encrypted data, since
     * CRYPTO-C will strip the padding. But we do not know how many, so for
     * simplicity, make the buffer the same size.
     */
    decryptedDataLen = encryptedDataLen;
    decryptedData = T_malloc (decryptedDataLen);
    if ((status = (decryptedData == NULL_PTR)) != 0)
      break;
    
    if ((status = B_DecryptUpdate
         (rc2Decrypter, decryptedData, &decryptedLenUpdate,
          decryptedDataLen, encryptedData, outputLenTotal,
          (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    
    if ((status = B_DecryptFinal
         (rc2Decrypter, decryptedData + decryptedLenUpdate,
          &decryptedLenFinal, decryptedDataLen - decryptedLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    decryptedLenTotal = decryptedLenUpdate + decryptedLenFinal;
    printf ("Decrypted data in hex (%u bytes):\n", decryptedLenTotal);
    PrintBuf (decryptedData, decryptedLenTotal);
    
    printf ("Decrypted data: %s \n", decryptedData);
  
  } while (0);

  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Encrypting/Decrypting with RC2 in CBC mode failed");
  }  

  /*  Done with the key and algorithm objects, so destroy them.
   */
  B_DestroyKeyObject (&rc2Key);
  B_DestroyAlgorithmObject (&rc2Encrypter);
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&rc2Decrypter);

  /*  Free up any memory allocated, save it to a file or print it out first
      if you need to save it.
   */
  if (randomSeed != NULL_PTR) {
    T_memset (randomSeed, 0, randomSeedLen);
    T_free (randomSeed);
    randomSeed = NULL_PTR;
  }

  if (encryptedData != NULL_PTR) {
    T_memset (encryptedData, 0, encryptedDataLen);
    T_free (encryptedData);
    encryptedData = NULL_PTR;
  }
  
  if (decryptedData != NULL_PTR) {
    T_memset (decryptedData, 0, decryptedDataLen);
    T_free (decryptedData);
    decryptedData = NULL_PTR;
  }
  
  if (rc2KeyItem.data != NULL_PTR) {
    T_memset (rc2KeyItem.data, 0, rc2KeyItem.len);
    T_free (rc2KeyItem.data);
    rc2KeyItem.data = NULL_PTR;
    rc2KeyItem.len = 0;
  }
  
} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
