/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program will generate Diffie-Hellman parameters needed for
      the key agreement (see dhagree.c).
 */

#include <stdio.h>   
#include <string.h>  
#include <time.h>
#include "aglobal.h"
#include "bsafe.h"

B_ALGORITHM_METHOD *DH_SAMPLE_CHOOSER[] = {
  &AM_SHA_RANDOM,
  &AM_DH_PARAM_GEN,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

int GeneralSurrenderFunction PROTO_LIST ((POINTER handle));

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ dhParamGenerator = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ dhParametersObj = (B_ALGORITHM_OBJ)NULL_PTR;

  A_DH_PARAM_GEN_PARAMS dhParams;

  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;
  
  A_SURRENDER_CTX generalSurrenderContext;

  ITEM *cryptocDHParametersBER;
  ITEM myDHParametersBER;

  unsigned int status, generalFlag;

  generalSurrenderContext.Surrender = GeneralSurrenderFunction;
  generalSurrenderContext.handle = (POINTER)&generalFlag;
  generalSurrenderContext.reserved = NULL_PTR;
    
  myDHParametersBER.data = NULL_PTR;

  do {
    printf ("Diffie-Hellman Algorithm \n");
    printf ("======================== \n");

    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.  
     */
    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, DH_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
      break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
        (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;
                
    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    /*  ========================================================  */
    
    printf ("\n   Generating DH parameters -- will take awhile\n");
    printf ("   ============================================ \n");

    /*  Step 1:  Create algorithm object */
    if ((status = B_CreateAlgorithmObject (&dhParamGenerator)) != 0)
      break;

    /*  Step 2:  Set algorithm object to AI_DHParamGen
     */
    dhParams.primeBits = 512;
    dhParams.exponentBits = 504;
    if ((status = B_SetAlgorithmInfo
         (dhParamGenerator, AI_DHParamGen,
          (POINTER)&dhParams)) != 0)
      break;

    /*  Step 3:  Init */
    if ((status = B_GenerateInit
         (dhParamGenerator, DH_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  no step 4 (Update) in generating DH parameters */

    /*  Step 5:  Generate DH parameters
     */
    if ((status = B_CreateAlgorithmObject (&dhParametersObj)) != 0)
      break;

    /*  generalFlag is for the surrender function. */
    generalFlag = 0;
    if ((status = B_GenerateParameters
         (dhParamGenerator, dhParametersObj, randomAlgorithm,
          &generalSurrenderContext)) != 0)
      break;

    printf ("\n   Distributing DH parameters \n");
    printf ("   ========================== \n");

    if ((status = B_GetAlgorithmInfo
         ((POINTER *)&cryptocDHParametersBER, dhParametersObj,
          AI_DHKeyAgreeBER)) != 0)
      break;

    myDHParametersBER.len = cryptocDHParametersBER->len;
    myDHParametersBER.data = T_malloc (myDHParametersBER.len);
    if ((status = (myDHParametersBER.data == NULL_PTR)) != 0)
      break;
    T_memcpy (myDHParametersBER.data, cryptocDHParametersBER->data,
              myDHParametersBER.len);

    printf ("The DH parameters (%u bytes):\n", myDHParametersBER.len);
    PrintBuf (myDHParametersBER.data, myDHParametersBER.len);

  } while (0);
    
  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("DH Parameter Generation failed");
  }  
  
  /*  Step 6:  Destroy
   */
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&dhParametersObj);
  B_DestroyAlgorithmObject (&dhParamGenerator);

  /*  Free up any memory allocated
   */
  T_free (randomSeed);
  T_free (myDHParametersBER.data);

} /*  end main  */

/*   General Surrender Function   */
/*   ==========================   */
int GeneralSurrenderFunction (handle)
POINTER handle;
{
  char s[100];
  static time_t currentTime;
  time_t getTime;

  if ((int)*handle == 0) {
    getTime = time(NULL);
    strftime (s, 100, "%H:%M:%S on %A, %d %B %Y", localtime(&getTime));
    printf ("\n%s\n", s);
    printf ("Surrender function ...\n");
    *handle = 1;
    time (&currentTime);
  }
  else {
    time (&getTime);
    if (currentTime != getTime) {
      printf (" .");
      currentTime = getTime;
    }
  }
  return (0);
} /*  end GeneralSurrenderFunction  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
