/* Copyright (C) RSA Data Security, Inc. created 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program will generate a RSA keypair to perform signing
 *  and verifying according to the X9.31 standard.
 */
     
#include <stdio.h>    
#include <string.h>   
#include <time.h>
#include "aglobal.h"
#include "bsafe.h"

B_ALGORITHM_METHOD *SIGNVERIFY_SAMPLE_CHOOSER[] = {
  &AM_SHA,
  &AM_FORMAT_X931,
  &AM_RSA_CRT_X931_ENCRYPT,
  &AM_EXTRACT_X931,
  &AM_RSA_X931_DECRYPT,
  &AM_RSA_STRONG_KEY_GEN,
  &AM_SHA_RANDOM,
  /*  &AM_MD5_RANDOM, */
  (B_ALGORITHM_METHOD *)NULL_PTR
};

#define RSA_MODULUS_BITS 512

int GeneralSurrenderFunction PROTO_LIST ((POINTER handle));

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ keypairGenerator = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ digitalSigner = (B_ALGORITHM_OBJ)NULL_PTR;  
  B_ALGORITHM_OBJ digitalVerifier = (B_ALGORITHM_OBJ)NULL_PTR;
  B_KEY_OBJ publicKey = (B_KEY_OBJ)NULL_PTR;
  B_KEY_OBJ privateKey = (B_KEY_OBJ)NULL_PTR;

  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;
  
  A_RSA_KEY_GEN_PARAMS keygenParams;
  A_X931_PARAMS x931params;
  B_SIGN_VERIFY_PARAMS signVerifyParams;
  
  static unsigned char f4Data[] = {0x01, 0x00, 0x01};
  
  A_SURRENDER_CTX generalSurrenderContext;
  int generalFlag;

  char *inputData = "Sign this sentence.";
  unsigned int inputDataLen;

  unsigned char signature[64];
  unsigned int signatureLen;

  unsigned int status;

  generalSurrenderContext.Surrender = GeneralSurrenderFunction;
  generalSurrenderContext.handle = (POINTER)&generalFlag;
  generalSurrenderContext.reserved = NULL_PTR;

  do {
    printf ("Digital Signature\n");
    printf ("=================\n");
    
    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */
    printf ("  Generating random bytes \n");
    printf ("  ======================= \n");

    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, SIGNVERIFY_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
         break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  ========================================================  */
    /*  The following code is pulled from rsapkcs.c, steps 1-5 to
          generate a RSA keypair.
     */

    printf ("\n   Generating a Keypair \n");
    printf ("   ==================== \n");

    if ((status = B_CreateAlgorithmObject (&keypairGenerator)) != 0)
      break;

    keygenParams.modulusBits = RSA_MODULUS_BITS;
    keygenParams.publicExponent.data = f4Data;
    keygenParams.publicExponent.len = 3;
    
    if ((status = B_SetAlgorithmInfo (keypairGenerator, AI_RSAStrongKeyGen,
                                      (POINTER)&keygenParams)) != 0)
      break;

    if ((status = B_GenerateInit
         (keypairGenerator, SIGNVERIFY_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    if ((status = B_CreateKeyObject (&publicKey)) != 0)
      break;
    if ((status = B_CreateKeyObject (&privateKey)) != 0)
      break;

    /*  generalFlag is for the surrender function */
    generalFlag = 0;
    if ((status = B_GenerateKeypair
         (keypairGenerator, publicKey, privateKey,
          randomAlgorithm, &generalSurrenderContext)) != 0)
      break;
    
    printf ("\nComputing a Digital Signature \n");
    printf ("============================= \n");

    printf ("Input data: %s \n", inputData);
    inputDataLen = (unsigned int)strlen (inputData);
    printf ("Input data in hex: \n");
    PrintBuf ((unsigned char *)inputData, inputDataLen);
  
    /*  Step 1:  Create an algorithm object */
    if ((status = B_CreateAlgorithmObject (&digitalSigner)) != 0)
      break;

    /*  Step 2:  Set the algorithm object to AI_SignVerify */
    x931params.blockLen = ((RSA_MODULUS_BITS + 7) / 8);
    x931params.oidNum = 3;
    
    signVerifyParams.encryptionMethodName = (unsigned char *)"rsaSignX931";
    signVerifyParams.encryptionParams = NULL_PTR;
    signVerifyParams.digestMethodName = (unsigned char *)"sha1";
    signVerifyParams.digestParams = NULL_PTR;
    signVerifyParams.formatMethodName = (unsigned char *)"formatX931";
    signVerifyParams.formatParams = (POINTER)&x931params;
    
    if ((status = B_SetAlgorithmInfo (digitalSigner, AI_SignVerify,
                                      (POINTER)&signVerifyParams)) != 0)
      break;

    /*  Step 3:  Init */
    if ((status = B_SignInit
         (digitalSigner, privateKey, SIGNVERIFY_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update  -- digest the data to sign */
    printf (".......Digesting the input data\n");
    if ((status = B_SignUpdate
         (digitalSigner, (unsigned char *)inputData, inputDataLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 5:  Final --  Encrypt the digest and output the result to a
                           signature buffer */
    printf (".......Encrypting the digest (digital signature)\n");
    if ((status = B_SignFinal
         (digitalSigner, signature, &signatureLen, 64,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("\nThe digital signature (%u bytes): \n", signatureLen);
    PrintBuf (signature, signatureLen);
    
    printf ("Verifying the Digital Signature \n");
    printf ("=============================== \n");

    /*  Step 1:  Create an algorithm object */
    if ((status = B_CreateAlgorithmObject (&digitalVerifier)) != 0)
      break;

    /*  Step 2:  Set the algorithm object to the same AI */
    signVerifyParams.encryptionMethodName = (unsigned char *)"rsaVerifyX931";
    signVerifyParams.encryptionParams = NULL_PTR;
    signVerifyParams.digestMethodName = (unsigned char *)"sha1";
    signVerifyParams.digestParams = NULL_PTR;
    signVerifyParams.formatMethodName = (unsigned char *)"formatX931";
    signVerifyParams.formatParams = (POINTER)&x931params;
    
    if ((status = B_SetAlgorithmInfo (digitalVerifier, AI_SignVerify,
                                      (POINTER)&signVerifyParams)) != 0)
      break;

    /*  Step 3:  Init */
    if ((status = B_VerifyInit
         (digitalVerifier, publicKey, SIGNVERIFY_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update */
    if ((status = B_VerifyUpdate
         (digitalVerifier, (unsigned char *)inputData, inputDataLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

     /*  Step 5:  Final */
    generalFlag = 0;
    if ((status = B_VerifyFinal
         (digitalVerifier, signature, signatureLen,
          (B_ALGORITHM_OBJ)NULL_PTR,
          &generalSurrenderContext)) != 0)
      break;

  } while (0);
    
  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Digital Signature failed");
  }
  else {
    printf ("\nDigital Signature verified.");
  }  
  
  /*  Step 6:  Destroy
   */
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&keypairGenerator);
  B_DestroyAlgorithmObject (&digitalSigner);
  B_DestroyAlgorithmObject (&digitalVerifier);
  B_DestroyKeyObject (&privateKey);
  B_DestroyKeyObject (&publicKey);

  /*  Free up any memory allocated */
  T_free (randomSeed);

} /*  end main  */

/*   General Surrender Function   */
/*   ==========================   */
int GeneralSurrenderFunction (handle)
POINTER handle;
{
  char s[100];
  static time_t currentTime;
  time_t getTime;

  if ((int)*handle == 0) {
    getTime = time(NULL);
    strftime (s, 100, "%H:%M:%S on %A, %d %B %Y", localtime(&getTime));
    printf ("\n%s\n", s);
    printf ("Surrender function ...\n");
    *handle = 1;
    time (&currentTime);
  }
  else {
    time (&getTime);
    if (currentTime != getTime) {
      printf (" .");
      currentTime = getTime;
    }
  }
  return (0);
} /*  end GeneralSurrenderFunction  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
