/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program will convert data between Binary and ASCII.
 */
     
#include <stdio.h>    
#include <string.h>   
#include <time.h>
#include "aglobal.h"
#include "bsafe.h"

B_ALGORITHM_METHOD *ENCDEC_SAMPLE_CHOOSER[] = {
  &AM_SHA_RANDOM, 
  (B_ALGORITHM_METHOD *)NULL_PTR
};

#define NUMBER_OF_RANDOM_BYTES 64

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ asciiEncoder = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ asciiDecoder = (B_ALGORITHM_OBJ)NULL_PTR;

  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;

  unsigned char *binaryData = NULL_PTR;
  unsigned int binaryDataLen;

  unsigned char *asciiEncoding = NULL_PTR;
  unsigned int asciiEncodingLenUpdate;
  unsigned int asciiEncodingLenFinal;
  unsigned int asciiEncodingLenTotal;

  unsigned char *binaryDecoding = NULL_PTR;
  unsigned int binaryDecodingLenUpdate;
  unsigned int binaryDecodingLenFinal;
  unsigned int binaryDecodingLenTotal;
  
  unsigned int i, status;

  do {
    printf ("Encoding-Decoding Algorithm \n");
    printf ("=========================== \n");

    printf ("  First, generate some random bytes to be encoded \n");

    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */
    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, ENCDEC_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
      break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    /*  ========================================================  */

    binaryData = T_malloc (NUMBER_OF_RANDOM_BYTES);
    if ((status = (binaryData == NULL_PTR)) != 0)
      break;

    if ((status = B_GenerateRandomBytes
         (randomAlgorithm, binaryData, NUMBER_OF_RANDOM_BYTES,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("\nThe generated random bytes ... \n");
    binaryDataLen = NUMBER_OF_RANDOM_BYTES;
    PrintBuf (binaryData, binaryDataLen);

    printf ("Now, encode the random bytes to ASCII ... \n");
    
    /*  Step 1:  Create algorithm object */
    if ((status = B_CreateAlgorithmObject (&asciiEncoder)) != 0)
      break;

    /*  Step 2:  Set algorithm object to AI_RFC1113Recode */
    if ((status = B_SetAlgorithmInfo
         (asciiEncoder, AI_RFC1113Recode, NULL_PTR)) != 0)
      break;

    /*  Step 3:  Init */
    if ((status = B_EncodeInit (asciiEncoder)) != 0)
      break;

    /*  Step 4:  Update & Final -- Encoding stage
     */
    asciiEncoding = T_malloc (binaryDataLen * 2);
    if ((status = (asciiEncoding == NULL_PTR)) != 0)
      break;

    if ((status = B_EncodeUpdate
         (asciiEncoder, asciiEncoding, &asciiEncodingLenUpdate,
          (binaryDataLen * 2), binaryData, binaryDataLen)) != 0)
      break;

    if ((status = B_EncodeFinal
         (asciiEncoder, asciiEncoding + asciiEncodingLenUpdate,
          &asciiEncodingLenFinal,
          (binaryDataLen * 2) - asciiEncodingLenUpdate)) != 0)
      break;

    asciiEncodingLenTotal = asciiEncodingLenUpdate + asciiEncodingLenFinal;
    
    printf ("The ASCII-encoded data ... \n");
    asciiEncoding[asciiEncodingLenTotal] = 0;
    
    for (i = 0; i < (unsigned int)asciiEncodingLenTotal; ++i) {
      putchar ((int)asciiEncoding[i]);
      if ( ((i & 31) == 31) ||
           (i == (unsigned int)(asciiEncodingLenTotal - 1)) )
        printf ("\n  ");
    }

    printf ("\nNow, decode the ASCII-encoded data ... \n");

    if ((status = B_CreateAlgorithmObject (&asciiDecoder)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (asciiDecoder, AI_RFC1113Recode, NULL_PTR)) != 0)
      break;

    if ((status = B_DecodeInit (asciiDecoder)) != 0)
      break;

    binaryDecoding = T_malloc (asciiEncodingLenTotal);
    if ((status = (binaryDecoding == NULL_PTR)) != 0)
      break;
    
    if ((status = B_DecodeUpdate
         (asciiDecoder, binaryDecoding, &binaryDecodingLenUpdate,
          asciiEncodingLenTotal, asciiEncoding,
          asciiEncodingLenTotal)) != 0)
      break;

    if ((status = B_DecodeFinal
         (asciiDecoder, binaryDecoding + binaryDecodingLenUpdate,
          &binaryDecodingLenFinal,
          (asciiEncodingLenTotal - binaryDecodingLenUpdate))) != 0)
      break;

    binaryDecodingLenTotal = binaryDecodingLenUpdate + binaryDecodingLenFinal;
    
    printf ("The Binary-decoded data ... \n");
    PrintBuf (binaryDecoding, binaryDecodingLenTotal);

    if ((status = T_memcmp (binaryData, binaryDecoding,
                            binaryDecodingLenTotal)) != 0)
      break;

    printf("The binary-decoded data matches the original binary data.\n");
    
  } while (0);
        
  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Converting data between Binary and ASCII failed");
  }  
 
  /*  Step 6:  Destroy
   */
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&asciiEncoder);           
  B_DestroyAlgorithmObject (&asciiDecoder);

  /*  Free up any memory allocated
   */
  T_free (randomSeed);
  T_free (binaryData);
  T_free (asciiEncoding);
  T_free (binaryDecoding);

} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
