/* Copyright (C) RSA Data Security, Inc. created 1995, modified 1997.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This is the Introductory Example program.
 */

#include <stdio.h>   
#include <string.h>  
#include "aglobal.h"
#include "bsafe.h"

B_ALGORITHM_METHOD *FEEDBACK_CHOOSER[] = {
  &AM_CBC_ENCRYPT,
  &AM_CBC_DECRYPT,
  &AM_DES_ENCRYPT,
  &AM_DES_DECRYPT,
  &AM_SHA_RANDOM,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_KEY_OBJ desKey = (B_KEY_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ encryptionObject = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ decryptionObject = (B_ALGORITHM_OBJ)NULL_PTR;
  
  unsigned char initVector[] = {
    0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08
  };
  unsigned char desKeyData[] = {
    0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88
  };

  ITEM ivItem;

  B_BLK_CIPHER_W_FEEDBACK_PARAMS fbParams;

  static char *dataToEncrypt = "Encrypt this sentence.";
  unsigned int dataToEncryptLen, encryptedDataLen, decryptedDataLen;
  unsigned int outputLenUpdate, outputLenFinal;

  unsigned char *encryptedData = NULL_PTR;
  unsigned char *decryptedData = NULL_PTR;
  /* outputLenTotal is the sum of the encryptions outputLenUpdate
     and outputLenFinal.  The encrypter should send this information
     along with the encrypted data. */
  unsigned int outputLenTotal;
  unsigned int status;

  do {
    /*  Find out how many bytes to encrypt. */
    dataToEncryptLen = (unsigned int)strlen (dataToEncrypt);
  
    printf ("Feedback Cipher:  DES Encryption/Decryption \n");
    printf ("=========================================== \n");
    printf ("\n");

    printf ("Encryption phase: \n");
    printf ("================= \n");
    printf ("Input data: %s \n", dataToEncrypt);
    printf ("\nInput data in hex: \n");
    PrintBuf ((unsigned char *)dataToEncrypt, dataToEncryptLen);

    /*  Allocate a buffer that is at least 8 bytes longer than the
          length of the input buffer to hold padding.  */
    encryptedDataLen = dataToEncryptLen + 8;
    encryptedData = T_malloc (encryptedDataLen);
    if ((status = (encryptedData == NULL_PTR)) != 0)
      break;
    
    /*  Create a key object. */
    if ((status = B_CreateKeyObject (&desKey)) != 0)
      break;

    /*  Set the key to be a DES key and set it to the desKeyData declared
        above. */
    if ((status = B_SetKeyInfo
         (desKey, KI_DES8Strong, (unsigned char *)desKeyData)) != 0)
      break;

    /*  Create an algorithm object. */
    if ((status = B_CreateAlgorithmObject (&encryptionObject)) != 0)
      break;

    /*  Set the algorithm to a type that does DES encryption in CBC mode. 
     */
    ivItem.data = initVector;
    ivItem.len = 8;
    
    fbParams.encryptionMethodName = (unsigned char *)"des";
    fbParams.encryptionParams = NULL_PTR;
    fbParams.feedbackMethodName = (unsigned char *)"cbc";
    fbParams.feedbackParams = (POINTER)&ivItem;
    fbParams.paddingMethodName = (unsigned char *)"pad";
    fbParams.paddingParams = NULL_PTR;

    if ((status = B_SetAlgorithmInfo
         (encryptionObject, AI_FeedbackCipher, (POINTER)&fbParams)) != 0)
      break;

    /*  Ready to encrypt. Use B_EncryptInit, B_EncryptUpdate
        and B_EncryptFinal.
     */
    if ((status = B_EncryptInit
         (encryptionObject, desKey, FEEDBACK_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    if ((status = B_EncryptUpdate
         (encryptionObject, encryptedData, &outputLenUpdate,
          encryptedDataLen, (unsigned char *)dataToEncrypt,
          dataToEncryptLen, (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("outputLenUpdate = %i \n", outputLenUpdate);

    if ((status = B_EncryptFinal
         (encryptionObject, encryptedData + outputLenUpdate,
          &outputLenFinal, encryptedDataLen - outputLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("outputLenFinal = %i \n", outputLenFinal);
    outputLenTotal = outputLenUpdate + outputLenFinal;
    printf ("Total length of encrypted data = %i \n", outputLenTotal);

    printf ("Encrypted data: \n");
    PrintBuf (encryptedData, (outputLenUpdate + outputLenFinal));

    printf ("\n");
    printf ("Decryption phase \n");
    printf ("================ \n");

    if ((status = B_CreateAlgorithmObject 
	 (&decryptionObject)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
	 (decryptionObject, AI_FeedbackCipher, (POINTER)&fbParams)) != 0)
      break;

    if ((status = B_DecryptInit
	 (decryptionObject, desKey, FEEDBACK_CHOOSER,
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    decryptedDataLen = encryptedDataLen;
    decryptedData = T_malloc (decryptedDataLen);
    if ((status = (decryptedData == NULL_PTR)) != 0)
      break;

    if ((status = B_DecryptUpdate
	 (decryptionObject, decryptedData, &outputLenUpdate,
	  decryptedDataLen, encryptedData, outputLenTotal,
	  (B_ALGORITHM_OBJ)NULL_PTR,
	  (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("outputLenUpdate = %i \n", outputLenUpdate);
    
    if ((status = B_DecryptFinal
	 (decryptionObject, decryptedData + outputLenUpdate,
	  &outputLenFinal, decryptedDataLen - outputLenUpdate,
	  (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("outputLenFinal = %i \n", outputLenFinal);
    outputLenTotal = outputLenUpdate + outputLenFinal;
    printf ("Total length of decrypted data = %i \n", outputLenTotal);

    printf ("Decrypted data in hex: \n");
    PrintBuf (decryptedData, outputLenTotal);

    decryptedData[outputLenTotal] = '\0';
    printf ("Decrypted data (after placing the NULL terminating\n");
    printf ("character at the end): %s \n", decryptedData);

  } while (0);

  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("The Feedback Cipher Example failed");
  }
   
  /*  Done with the key and algorithm objects, so destroy them.
   */
  B_DestroyKeyObject (&desKey);
  B_DestroyAlgorithmObject (&encryptionObject);
  B_DestroyAlgorithmObject (&decryptionObject);

  /*  Free up any memory allocated and zeroize any sensitive memory.
        Save it to a file or print it out first if you need to save it.
   */
  if (encryptedData != NULL_PTR) {
    T_memset (encryptedData, 0, encryptedDataLen);
    T_free (encryptedData);
    encryptedData = NULL_PTR;
  }

  if (decryptedData != NULL_PTR) {
    T_memset (decryptedData, 0, decryptedDataLen);
    T_free (decryptedData);
    decryptedData = NULL_PTR;
  }
  
} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
}
