/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This is a Password-Based Encryption program.
 */

#include <stdio.h>   
#include <string.h>  
#include "aglobal.h"
#include "bsafe.h"

#define SALT_LEN 8
#define MAX_PW_LEN 20
#define BLOCK_SIZE 8

B_ALGORITHM_METHOD *PBE_CHOOSER[] = {
  &AM_MD5,
  &AM_SHA_RANDOM,
  &AM_RC2_CBC_ENCRYPT,
  &AM_RC2_CBC_DECRYPT,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_KEY_OBJ pbeKey = (B_KEY_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ pbEncrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ pbDecrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;

  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;

  B_RC2_PBE_PARAMS rc2PBEParams;
  unsigned char saltData[SALT_LEN];

  unsigned char enteredPassword[MAX_PW_LEN];
  ITEM pbeKeyItem;

  unsigned char dataToEncrypt[] = "Encrypt this sentence.";
  unsigned int dataToEncryptLen;
  unsigned char *encryptedData = NULL_PTR;
  unsigned int outputLenUpdate, outputLenFinal;
  unsigned int encryptedDataLen, decryptedDataLen;

  unsigned char *decryptedData = NULL_PTR;
  unsigned int decryptedLenUpdate, decryptedLenFinal;
  unsigned int decryptedLenTotal;
    
  unsigned int status;
  
  do {
    printf ("Password-Based algorithm:  Encryption phase \n");
    printf ("=========================================== \n");

    printf ("Input data: %s \n", dataToEncrypt);
    dataToEncryptLen = strlen ((char *)dataToEncrypt) + 1;
    printf ("Input data in hex: \n");
    PrintBuf ((unsigned char *)dataToEncrypt, dataToEncryptLen); 

    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */

    printf ("Generating random bytes \n");
    printf ("======================= \n");

    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, PBE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
         break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  ========================================================  */

    /*  Generate the bytes to get an 8-byte random "salt" */
        if ((status = B_GenerateRandomBytes
             (randomAlgorithm, saltData, SALT_LEN,
              (A_SURRENDER_CTX *)NULL_PTR)) != 0)
          break;
        
    /*  Step 1:  Create an algorithm object. */
    if ((status = B_CreateAlgorithmObject (&pbEncrypter)) != 0)
      break;

    /*  Step 2:  Set the algorithm to AI_MD5WithRC2_CBCPad.
     */
    rc2PBEParams.effectiveKeyBits = 80;
    rc2PBEParams.salt = saltData;
    rc2PBEParams.iterationCount = 5;

    printf ("\nHere's the %i-byte random ""salt"":\n", BLOCK_SIZE);
    PrintBuf (saltData, BLOCK_SIZE);
            
    if ((status = B_SetAlgorithmInfo
         (pbEncrypter, AI_MD5WithRC2_CBCPad,
          (POINTER)&rc2PBEParams)) != 0)
       break;

    /*  Step 3:  Init -- A key is needed before the algorithm object can
                 be initialized for encryption.  In PBE, the password is
                 the key.
     */
       /*  Step 3a:  Create a key object. */
       if ((status = B_CreateKeyObject (&pbeKey)) != 0)
         break;
        
       /*  Step 3b:  Set the key object -- The data portion of the struct
                     is the password.  An unsecure method for entering the
                     password would be the following.  This is simply for
                     illustrative purposes, not for duplication.
        */
        puts ("Enter the password, then press Return or Enter");
        if ((status =
	     (NULL_PTR == (unsigned char *)gets ((char *)enteredPassword))) != 0)
          break;
        pbeKeyItem.data = enteredPassword;
        pbeKeyItem.len = strlen ((char *)enteredPassword);

        if ((status = B_SetKeyInfo
             (pbeKey, KI_Item, (POINTER)&pbeKeyItem)) != 0)
          break;

        /*  Let's zeroize the memory and free it up immediately after
              setting the key for security reasons.
         */
        T_memset (pbeKeyItem.data, 0, MAX_PW_LEN);

    if ((status = B_EncryptInit
         (pbEncrypter, pbeKey, PBE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update
     */
    encryptedDataLen = dataToEncryptLen + BLOCK_SIZE;
    encryptedData = T_malloc (encryptedDataLen);
    if ((status = (encryptedData == NULL_PTR)) != 0)
      break;

    if ((status = B_EncryptUpdate
         (pbEncrypter, encryptedData, &outputLenUpdate,
          encryptedDataLen, dataToEncrypt, dataToEncryptLen,
	  (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 5:  Final
     */
    if ((status = B_EncryptFinal
         (pbEncrypter, encryptedData + outputLenUpdate,
          &outputLenFinal, encryptedDataLen - outputLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    encryptedDataLen = outputLenUpdate + outputLenFinal;
    printf ("Encrypted data: \n");
    PrintBuf (encryptedData, encryptedDataLen);
    
    /* Destroy the key object now, rebuild a new one for decryption. */
    B_DestroyKeyObject (&pbeKey);

    printf ("Password-Based algorithm:  Decryption phase \n");
    printf ("=========================================== \n");

    if ((status = B_CreateAlgorithmObject (&pbDecrypter)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (pbDecrypter, AI_MD5WithRC2_CBCPad,
          (POINTER)&rc2PBEParams)) != 0)
      break;

    /* Build a key from a new password. If it is the same password (and
         salt), the key will work
     */
    if ((status = B_CreateKeyObject (&pbeKey)) != 0)
      break;

    puts ("Enter the password for decryption.");
    puts (" [What happens if you enter the wrong password?]");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)enteredPassword))) != 0)
      break;
    pbeKeyItem.data = enteredPassword;
    pbeKeyItem.len = strlen ((char *)pbeKeyItem.data);

    if ((status = B_SetKeyInfo
         (pbeKey, KI_Item, (POINTER)&pbeKeyItem)) != 0)
      break;

    /*  Let's zeroize the memory and free it up immediately after
          setting the key for security reasons.
     */
    T_memset (pbeKeyItem.data, 0, MAX_PW_LEN);

    if ((status = B_DecryptInit
         (pbDecrypter, pbeKey, PBE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    decryptedDataLen = encryptedDataLen;
    decryptedData = T_malloc (decryptedDataLen);
    if ((status = (decryptedData == NULL_PTR)) != 0)
      break;

    /*  encryptedDataLen is the sum of the encryption's outputLenUpdate and
          outputLenFinal. The encrypter should send this information along
          with the encrypted data. This was calculated before during the
          encryption phase.
     */
    if ((status = B_DecryptUpdate
         (pbDecrypter, decryptedData, &decryptedLenUpdate,
          decryptedDataLen, encryptedData, encryptedDataLen,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    if ((status = B_DecryptFinal
         (pbDecrypter, decryptedData + decryptedLenUpdate,
          &decryptedLenFinal, decryptedDataLen - decryptedLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    decryptedLenTotal = decryptedLenUpdate + decryptedLenFinal;
    printf ("Decrypted data in hex: \n");
    PrintBuf (decryptedData, decryptedLenTotal);

    printf ("Decrypted data: %s \n", decryptedData);
  
  } while (0);

  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Password-Based Encryption failed. ");
  }
  
  /*  Done with the key and algorithm objects, so destroy them.
   */
  B_DestroyKeyObject (&pbeKey);
  B_DestroyAlgorithmObject (&pbEncrypter);
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&pbDecrypter);

  /*  Free up any memory allocated, save it to a file or print it out first
      if you need to save it.
   */
  if (randomSeed != NULL_PTR) {
    T_memset (randomSeed, 0, randomSeedLen);
    T_free (randomSeed);
    randomSeed = NULL_PTR;
  }

  if (encryptedData != NULL_PTR) {
    T_memset (encryptedData, 0, encryptedDataLen);
    T_free (encryptedData);
    encryptedData = NULL_PTR;
  }
  
  if (decryptedData != NULL_PTR) {
    T_memset (decryptedData, 0, decryptedDataLen);
    T_free (decryptedData);
    decryptedData = NULL_PTR;
  }

} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
