/* Copyright (C) RSA Data Security, Inc. created 1992.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

#include <stdio.h>
#include <string.h>
#include "aglobal.h"
#include "bsafe.h"
#include "demochos.h"
#include "bslite.h"
#include "filebsl.h"

/* If compiling with Borland, define BORLAND_STACK to the stack size,
     such as -DBORLAND_STACK=15000
 */
#ifdef BORLAND_STACK
extern unsigned _stklen = BORLAND_STACK;
#endif

/* If compiling with non-C calling format, define MAIN_CALL to force
     the compiler back to C calling format for main.  Otherwise, this
     sets MAIN_CALL to nothing.
 */
#ifndef MAIN_CALL
#define MAIN_CALL
#endif

int MAIN_CALL main PROTO_LIST ((int, char **));
static int SetOptions PROTO_LIST ((int, char **));
static void DoSignFile PROTO_LIST ((void));
static void DoSealFile PROTO_LIST ((void));
static void DoVerifyFile PROTO_LIST ((void));
static void DoOpenFile PROTO_LIST ((void));
static void DoGenerateKeys PROTO_LIST ((void));
static int GetPublicKey PROTO_LIST ((unsigned char **, unsigned int *,
  char *));
static int GetPrivateKey PROTO_LIST ((unsigned char **, unsigned int *,
  char *));
static void PrintMessage PROTO_LIST ((char *));
static void PrintError PROTO_LIST ((char *, int));
static int GetCommand PROTO_LIST ((char *, unsigned int, char *));
static char *BSAFE2_ErrorString PROTO_LIST ((int));

static int SILENT_PROMPT = 0;

static unsigned char PUBLIC_KEY1_DATA[] = {
  0x30, 0x5c, 0x30, 0x0d, 0x06, 0x09, 0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d,
  0x01, 0x01, 0x01, 0x05, 0x00, 0x03, 0x4b, 0x00, 0x30, 0x48, 0x02, 0x41,
  0x00, 0xc0, 0x76, 0x47, 0x97, 0xb8, 0xbe, 0xc8, 0x97, 0x2a, 0x0e, 0xd8,
  0xc9, 0x0a, 0x8c, 0x33, 0x4d, 0xd0, 0x49, 0xad, 0xd0, 0x22, 0x2c, 0x09,
  0xd2, 0x0b, 0xe0, 0xa7, 0x9e, 0x33, 0x89, 0x10, 0xbc, 0xae, 0x42, 0x20,
  0x60, 0x90, 0x6a, 0xe0, 0x22, 0x1d, 0xe3, 0xf3, 0xfc, 0x74, 0x7c, 0xcf,
  0x98, 0xae, 0xcc, 0x85, 0xd6, 0xed, 0xc5, 0x2d, 0x93, 0xd5, 0xb7, 0x39,
  0x67, 0x76, 0x16, 0x05, 0x25, 0x02, 0x03, 0x01, 0x00, 0x01
};

static unsigned char PRIVATE_KEY1_DATA[343] = {
  0x30, 0x82, 0x01, 0x53, 0x02, 0x01, 0x00, 0x30, 0x0d, 0x06, 0x09, 0x2a,
  0x86, 0x48, 0x86, 0xf7, 0x0d, 0x01, 0x01, 0x01, 0x05, 0x00, 0x04, 0x82,
  0x01, 0x3d, 0x30, 0x82, 0x01, 0x39, 0x02, 0x01, 0x00, 0x02, 0x41, 0x00,
  0xc0, 0x76, 0x47, 0x97, 0xb8, 0xbe, 0xc8, 0x97, 0x2a, 0x0e, 0xd8, 0xc9,
  0x0a, 0x8c, 0x33, 0x4d, 0xd0, 0x49, 0xad, 0xd0, 0x22, 0x2c, 0x09, 0xd2,
  0x0b, 0xe0, 0xa7, 0x9e, 0x33, 0x89, 0x10, 0xbc, 0xae, 0x42, 0x20, 0x60,
  0x90, 0x6a, 0xe0, 0x22, 0x1d, 0xe3, 0xf3, 0xfc, 0x74, 0x7c, 0xcf, 0x98,
  0xae, 0xcc, 0x85, 0xd6, 0xed, 0xc5, 0x2d, 0x93, 0xd5, 0xb7, 0x39, 0x67,
  0x76, 0x16, 0x05, 0x25, 0x02, 0x03, 0x01, 0x00, 0x01, 0x02, 0x40, 0x1a,
  0xe3, 0x6b, 0x75, 0x22, 0xf6, 0x64, 0x87, 0xd9, 0xf4, 0x61, 0x0d, 0x15,
  0x50, 0x29, 0x0a, 0xc2, 0x02, 0xc9, 0x29, 0xbe, 0xdc, 0x70, 0x32, 0xcc,
  0x3e, 0x02, 0xac, 0xf3, 0x7e, 0x3e, 0xbc, 0x1f, 0x86, 0x6e, 0xe7, 0xef,
  0x7a, 0x08, 0x68, 0xd2, 0x3a, 0xe2, 0xb1, 0x84, 0xc1, 0xab, 0xd6, 0xd4,
  0xdb, 0x8e, 0xa9, 0xbe, 0xc0, 0x46, 0xbd, 0x82, 0x80, 0x37, 0x27, 0xf2,
  0x88, 0x87, 0x01, 0x02, 0x21, 0x00, 0xdf, 0x02, 0xb6, 0x15, 0xfe, 0x15,
  0x92, 0x8f, 0x41, 0xb0, 0x2b, 0x58, 0x6b, 0x51, 0xc2, 0xc0, 0x22, 0x60,
  0xca, 0x39, 0x68, 0x18, 0xca, 0x4c, 0xba, 0x60, 0xbb, 0x89, 0x24, 0x65,
  0xbe, 0x35, 0x02, 0x21, 0x00, 0xdc, 0xee, 0xb6, 0x0d, 0x54, 0x35, 0x18,
  0xb4, 0xac, 0x74, 0x83, 0x4a, 0x05, 0x46, 0xc5, 0x07, 0xf2, 0xe9, 0x1e,
  0x38, 0x9a, 0x87, 0xe2, 0xf2, 0xbe, 0xcc, 0x6f, 0x8c, 0x67, 0xd1, 0xc9,
  0x31, 0x02, 0x20, 0x59, 0x48, 0x7e, 0x99, 0xe3, 0x75, 0xc3, 0x8d, 0x73,
  0x21, 0x12, 0xd9, 0x7d, 0x6d, 0xe8, 0x68, 0x7f, 0xda, 0xfc, 0x5b, 0x6b,
  0x5f, 0xb1, 0x6e, 0x72, 0x97, 0xd3, 0xbd, 0x1e, 0x43, 0x55, 0x99, 0x02,
  0x20, 0x61, 0xb5, 0x50, 0xde, 0x64, 0x37, 0x77, 0x4d, 0xb0, 0x57, 0x77,
  0x18, 0xed, 0x6c, 0x77, 0x07, 0x24, 0xee, 0xe4, 0x66, 0xb4, 0x31, 0x14,
  0xb5, 0xb6, 0x9c, 0x43, 0x59, 0x1d, 0x31, 0x32, 0x81, 0x02, 0x20, 0x74,
  0x4c, 0x79, 0xc4, 0xb9, 0xbe, 0xa9, 0x7c, 0x25, 0xe5, 0x63, 0xc9, 0x40,
  0x7a, 0x2d, 0x09, 0xb5, 0x73, 0x58, 0xaf, 0xe0, 0x9a, 0xf6, 0x7d, 0x71,
  0xf8, 0x19, 0x8c, 0xb7, 0xc9, 0x56, 0xb8
};

static unsigned char PUBLIC_KEY2_DATA[] = {
  0x30, 0x5c, 0x30, 0x0d, 0x06, 0x09, 0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d,
  0x01, 0x01, 0x01, 0x05, 0x00, 0x03, 0x4b, 0x00, 0x30, 0x48, 0x02, 0x41,
  0x00, 0xe5, 0xf2, 0x0d, 0xce, 0x83, 0x32, 0x30, 0xe4, 0x51, 0xd5, 0x0c,
  0x79, 0xa5, 0x34, 0x23, 0xfb, 0x5f, 0xfb, 0xa0, 0xff, 0x5c, 0x77, 0x96,
  0xa9, 0x2c, 0x35, 0xee, 0x53, 0x02, 0xb7, 0x89, 0x2d, 0xf0, 0x78, 0x49,
  0xdd, 0xdc, 0x27, 0x79, 0x8b, 0x69, 0xa2, 0x5e, 0x7a, 0x40, 0x27, 0x43,
  0x82, 0x7a, 0xc4, 0xfa, 0x50, 0x21, 0x1f, 0x15, 0x35, 0xc3, 0x93, 0x46,
  0x62, 0xcd, 0xd4, 0x26, 0x89, 0x02, 0x03, 0x01, 0x00, 0x01
};

static unsigned char PRIVATE_KEY2_DATA[345] = {
  0x30, 0x82, 0x01, 0x55, 0x02, 0x01, 0x00, 0x30, 0x0d, 0x06, 0x09, 0x2a,
  0x86, 0x48, 0x86, 0xf7, 0x0d, 0x01, 0x01, 0x01, 0x05, 0x00, 0x04, 0x82,
  0x01, 0x3f, 0x30, 0x82, 0x01, 0x3b, 0x02, 0x01, 0x00, 0x02, 0x41, 0x00,
  0xe5, 0xf2, 0x0d, 0xce, 0x83, 0x32, 0x30, 0xe4, 0x51, 0xd5, 0x0c, 0x79,
  0xa5, 0x34, 0x23, 0xfb, 0x5f, 0xfb, 0xa0, 0xff, 0x5c, 0x77, 0x96, 0xa9,
  0x2c, 0x35, 0xee, 0x53, 0x02, 0xb7, 0x89, 0x2d, 0xf0, 0x78, 0x49, 0xdd,
  0xdc, 0x27, 0x79, 0x8b, 0x69, 0xa2, 0x5e, 0x7a, 0x40, 0x27, 0x43, 0x82,
  0x7a, 0xc4, 0xfa, 0x50, 0x21, 0x1f, 0x15, 0x35, 0xc3, 0x93, 0x46, 0x62,
  0xcd, 0xd4, 0x26, 0x89, 0x02, 0x03, 0x01, 0x00, 0x01, 0x02, 0x41, 0x00,
  0x8e, 0x6b, 0x48, 0x4f, 0xe3, 0x04, 0x60, 0x31, 0xd9, 0xd9, 0x59, 0xdb,
  0xb2, 0x0d, 0xc4, 0x47, 0x7a, 0x60, 0x70, 0x8c, 0x19, 0x2a, 0x5c, 0x9f,
  0x35, 0x29, 0xc6, 0x0d, 0x95, 0xfe, 0x35, 0x30, 0x99, 0x54, 0xda, 0x99,
  0x90, 0xe3, 0x3e, 0x1d, 0xdb, 0x32, 0x7b, 0x1d, 0x26, 0xf2, 0x9a, 0xab,
  0x03, 0x09, 0x77, 0xba, 0x4f, 0x43, 0xdb, 0xda, 0xc8, 0x1f, 0x12, 0xd6,
  0xf2, 0x57, 0x1a, 0x31, 0x02, 0x21, 0x00, 0xf4, 0x6c, 0x5d, 0xb7, 0x7b,
  0xa7, 0x9e, 0x2f, 0x4a, 0x26, 0xec, 0xe0, 0x1b, 0x2a, 0x5f, 0x42, 0x04,
  0x87, 0xb4, 0xc4, 0x68, 0x80, 0xf3, 0x09, 0xe5, 0x48, 0x00, 0x30, 0x6f,
  0x54, 0x04, 0x75, 0x02, 0x21, 0x00, 0xf0, 0xd6, 0x25, 0x08, 0xe5, 0x54,
  0x19, 0xa3, 0x98, 0x9d, 0x77, 0x92, 0x18, 0xba, 0x9c, 0x00, 0xda, 0x69,
  0xee, 0x84, 0xf9, 0x10, 0xe5, 0xda, 0xf0, 0x7d, 0x78, 0x6c, 0x92, 0xd8,
  0xc7, 0x45, 0x02, 0x20, 0x62, 0x03, 0x32, 0x71, 0x69, 0x3c, 0x85, 0xca,
  0x20, 0xca, 0x0b, 0x22, 0xb2, 0xc2, 0x03, 0x00, 0xce, 0x20, 0x3c, 0xf4,
  0xa5, 0x93, 0x4f, 0xe2, 0x1f, 0xf7, 0x07, 0x5c, 0x64, 0xc8, 0x24, 0x21,
  0x02, 0x21, 0x00, 0xe4, 0xf1, 0xde, 0x01, 0x07, 0xe9, 0x1b, 0xfa, 0x7e,
  0x51, 0x6b, 0x62, 0xb2, 0x29, 0x6b, 0xb6, 0x0f, 0xeb, 0xd5, 0xf8, 0x08,
  0xc1, 0x7c, 0x90, 0xd6, 0x3f, 0xa0, 0xa4, 0x2e, 0x30, 0xef, 0xb5, 0x02,
  0x20, 0x3f, 0x01, 0x60, 0x46, 0x90, 0x85, 0xaf, 0x6f, 0xa2, 0x8d, 0x6a,
  0x1d, 0x3e, 0xfd, 0x56, 0xd7, 0x57, 0x5e, 0x00, 0x30, 0x5b, 0x49, 0x6c,
  0xc4, 0x85, 0xf5, 0x1c, 0x39, 0xef, 0x0b, 0x4b, 0x28
};

static unsigned char PUBLIC_KEY3_DATA[1024];
static unsigned int PUBLIC_KEY3_LEN = 0;
static unsigned char PRIVATE_KEY3_DATA[2048];
static unsigned int PRIVATE_KEY3_LEN = 0;

int MAIN_CALL main (argc, argv)
int argc;
char *argv[];
{
  char command[100];
  int done = 0;

  if (SetOptions (argc, argv))
    return (0);
  
  /* Print version message.
   */
  strcpy (command, "Command Line Demo for BSAFE version ");
  strcpy (command + strlen (command), BSAFE_VERSION);
  PrintMessage (command);

  PrintMessage
    ("NOTE: When saving to a file, a filename of \"-\" will output to the screen.");

  while (!done) {
    PrintMessage ("");
    PrintMessage ("S - Sign a file");
    PrintMessage ("E - Envelope a file");
    PrintMessage ("V - Verify a signed file");
    PrintMessage ("O - Open an enveloped file");
    PrintMessage ("G - Generate a keypair (may take a long time)");
    PrintMessage ("Q - Quit");
    GetCommand (command, sizeof (command), "  Enter choice: ");
    
    switch (*command) {
    case '\0':
    case '#':
      /* entered a blank line or a comment */
      break;
      
    case 's':
    case 'S':
      DoSignFile ();
      break;
      
    case 'e':
    case 'E':
      DoSealFile ();
      break;
      
    case 'v':
    case 'V':
      DoVerifyFile ();
      break;
      
    case 'o':
    case 'O':
      DoOpenFile ();
      break;
      
    case 'g':
    case 'G':
      DoGenerateKeys ();
      break;
      
    case 'Q':
    case 'q':
      done = 1;
      break;
      
    default:
      PrintError ("ERROR: Unrecognized command.  Try again.", 0);
      break;
    }
  }
  
  return (0);
}

/* Set options from command line and return 0 for success, 1 for bad format.
 */
static int SetOptions (argc, argv)
int argc;
char *argv[];
{
  int i, status = 0;
  
  for (i = 1; i < argc; i++) {
    if (argv[i][0] != '-') {
      status = 1;
      break;
    }
    
    if (argv[i][1] == 's')
      SILENT_PROMPT = 1;
    else {
      status = 1;
      break;
    }
  }

  if (status)
    puts ("Usage: bdemo [-s]\n  -s silent prompts");

  return (status);
}

static void DoSignFile ()
{
  char inputFilename[256], signatureFilename[256];
  int status;
  unsigned char *privateKey;
  unsigned int privateKeyLen;

  if (GetCommand (inputFilename, sizeof (inputFilename),
              "  Enter filename of content to sign (blank to cancel): "))
    return;
  if (GetCommand (signatureFilename, sizeof (signatureFilename),
              "  Enter filename to save the signature (blank to cancel): "))
    return;

  if ((*inputFilename == '\0') || (*signatureFilename == '\0'))
    return;
  
  if (GetPrivateKey
      (&privateKey, &privateKeyLen,
       "  Sign as user with private key 1, 2, or 3? (blank to cancel): "))
    return;

  if ((status = BSL_MD5WithRSASignFile
       (signatureFilename, inputFilename, privateKey, privateKeyLen,
        (unsigned char *)NULL_PTR, 0, (unsigned char *)NULL_PTR, 0)) != 0)
    PrintError ("signing file", status);
  else
    PrintMessage ("File Signed.");
}

static void DoSealFile ()
{
  char encryptedKeyFilename[256], ivFilename[256], outputFilename[256],
    inputFilename[256], seed[256];
  int status;
  unsigned char *publicKey;
  unsigned int publicKeyLen;

  if (GetCommand (inputFilename, sizeof (inputFilename),
       "  Enter filename of content to seal (blank to cancel): "))
    return;
  if (GetCommand (encryptedKeyFilename, sizeof (encryptedKeyFilename),
       "  Enter filename to save the encrypted key (blank to cancel): "))
    return;
  if (GetCommand (ivFilename, sizeof (ivFilename),
       "  Enter filename to save the key's IV (blank to cancel): "))
    return;
  if (GetCommand (outputFilename, sizeof (outputFilename),
       "  Enter filename to save the encrypted content (blank to cancel): "))
    return;
  if (GetCommand (seed, sizeof (seed),
       "  Enter seed to generate random DES key and IV (blank to cancel): "))
    return;

  if (GetPublicKey
      (&publicKey, &publicKeyLen, 
       "  Seal for user with public key 1, 2, or 3? (blank to cancel): "))
    return;
  
  if ((status = BSL_SealFileDESWithRSAEnvelope
       (encryptedKeyFilename, ivFilename, outputFilename, inputFilename,
        (unsigned char *)seed, (unsigned int)strlen (seed), publicKey,
        publicKeyLen)) != 0) {
    PrintError ("sealing file", status);
    return;
  }
  else
    PrintMessage ("Envelope Sealed.");
}

static void DoVerifyFile ()
{
  char inputFilename[256], signatureFilename[256];
  int status;
  unsigned char *publicKey;
  unsigned int publicKeyLen;
  
  if (GetCommand (inputFilename, sizeof (inputFilename),
              "  Enter name of file to verify (blank to cancel): "))
    return;
  if (GetCommand (signatureFilename, sizeof (signatureFilename),
              "  Enter filename of signature (blank to cancel): "))
    return;

  if ((*inputFilename == '\0') || (*signatureFilename == '\0'))
    return;
  
  if (GetPublicKey
      (&publicKey, &publicKeyLen,
       "  Verify signature from user with public key 1, 2, or 3? (blank to cancel): "))
    return;

  if ((status = BSL_MD5WithRSAVerifyFile
       (signatureFilename, inputFilename, publicKey,
        publicKeyLen)) != 0) {
    PrintError ("verifying file", status);
    return;
  }
  else
    PrintMessage ("Signature verified.");
}

static void DoOpenFile ()
{
  char resultDataFilename[256], encryptedKeyFilename[256], ivFilename[256],
    encryptedDataFilename[256];
  int status;
  unsigned char *privateKey;
  unsigned int privateKeyLen;

  if (GetCommand (encryptedDataFilename, sizeof (encryptedDataFilename),
       "  Enter filename of content to open (blank to cancel): "))
    return;
  if (GetCommand (encryptedKeyFilename, sizeof (encryptedKeyFilename),
       "  Enter filename of the encrypted key (blank to cancel): "))
    return;
  if (GetCommand (ivFilename, sizeof (ivFilename),
       "  Enter filename of the key's IV (blank to cancel): "))
    return;
  if (GetCommand (resultDataFilename, sizeof (resultDataFilename),
       "  Enter filename to save the decrypted content (blank to cancel): "))
    return;

  if (GetPrivateKey
      (&privateKey, &privateKeyLen,
       "  Open for user with private key 1, 2, or 3? (blank to cancel): "))
    return;

  if ((status = BSL_OpenFileDESWithRSAEnvelope
       (resultDataFilename, encryptedKeyFilename, ivFilename,
        encryptedDataFilename, privateKey, privateKeyLen, 0, 0, 0,
        0)) != 0) {
    PrintError ("opening enveloped file", status);
    return;
  }
  else
    PrintMessage ("Envelope opened.");
}

static void DoGenerateKeys ()
{
  char command[80];
  int status, modulusBits;
  char seed[256];

  if (PRIVATE_KEY3_LEN != 0) {
    PrintMessage
      ("Public key 3 and private key 3 have already been generated.");
    return;
  }
  
  if ((GetCommand
    (command, sizeof (command),
      "  Enter key size in bits, (360 to 2048) (blank to cancel): ")) != 0)
    return;

  sscanf (command, "%d", &modulusBits);

  if (modulusBits < 360 || modulusBits > 2048) {
    PrintError ("generating keys", BE_MODULUS_LEN);
    return;
  }

  if ((GetCommand (seed, sizeof (seed),
       "  Enter seed to generate the RSA keypair (blank to cancel): ")) != 0)
    return;

  PrintError ("  Generating RSA keypair...", 0);

  if ((status = BSL_MakeRSAKeyPairFromSeed
       (PUBLIC_KEY3_DATA, &PUBLIC_KEY3_LEN, PRIVATE_KEY3_DATA,
        &PRIVATE_KEY3_LEN, (unsigned char *)NULL_PTR, 0,
        (unsigned int)modulusBits, (unsigned char *)seed,
        (unsigned int)strlen (seed), (unsigned char *)NULL_PTR, 0)) != 0) {
    PrintError ("generating keys", status);
    return;
  }

  PrintMessage ("Public key 3 and private key 3 are now ready to use.");
}

/* Use the prompt to ask the user to use public key 1, 2 or 3 and
     point publicKey to the answer.
   Return 0 on success or 1 if user cancels by entering a blank.
 */
static int GetPublicKey (publicKey, publicKeyLen, prompt)
unsigned char **publicKey;
unsigned int *publicKeyLen;
char *prompt;
{
  char command[80];
  
  while (1) {
    GetCommand (command, sizeof (command), prompt);

    switch (*command) {
    case '\0':
      return (1);
      
    case '1':
      *publicKey = PUBLIC_KEY1_DATA;
      *publicKeyLen = sizeof (PUBLIC_KEY1_DATA);
      return (0);
      
    case '2':
      *publicKey = PUBLIC_KEY2_DATA;
      *publicKeyLen = sizeof (PUBLIC_KEY2_DATA);
      return (0);
      
    case '3':
      if (PUBLIC_KEY3_LEN == 0) {
        PrintError
          ("ERROR: Public key 3 has not been generated yet.  Try Again.", 0);
        break;
      }
      else {
        *publicKey = PUBLIC_KEY3_DATA;
        *publicKeyLen = PUBLIC_KEY3_LEN;
        return (0);
      }
      
    default:
      if (PUBLIC_KEY3_LEN != 0)
        PrintError ("ERROR: Please enter 1, 2 or 3.  Try again.", 0);
      else
        PrintError ("ERROR: Please enter 1 or 2.  Try again.", 0);
      break;
    }
  }
}

/* Use the prompt to ask the user to use private key 1, 2 or 3 and
     point privateKey to the answer.
   Return 0 on success or 1 if user cancels by entering a blank.
 */
static int GetPrivateKey (privateKey, privateKeyLen, prompt)
unsigned char **privateKey;
unsigned int *privateKeyLen;
char *prompt;
{
  char command[80];
  
  while (1) {
    GetCommand (command, sizeof (command), prompt);
    
    switch (*command) {
    case '\0':
      return (1);
      
    case '1':
      *privateKey = PRIVATE_KEY1_DATA;
      *privateKeyLen = sizeof (PRIVATE_KEY1_DATA);
      return (0);
      
    case '2':
      *privateKey = PRIVATE_KEY2_DATA;
      *privateKeyLen = sizeof (PRIVATE_KEY2_DATA);
      return (0);
      
    case '3':
      if (PRIVATE_KEY3_LEN == 0) {
        PrintError
          ("ERROR: Private key 3 has not been generated yet.  Try Again.", 0);
        break;
      }
      else {
        *privateKey = PRIVATE_KEY3_DATA;
        *privateKeyLen = PRIVATE_KEY3_LEN;
        return (0);
      }
      
    default:
      if (PRIVATE_KEY3_LEN != 0)
        PrintError ("ERROR: Please enter 1, 2 or 3.  Try again.", 0);
      else
        PrintError ("ERROR: Please enter 1 or 2.  Try again.", 0);
      break;
    }
  }
}

static void PrintMessage (message)
char *message;
{
  if (!SILENT_PROMPT) {
    puts (message);
    fflush (stdout);
  }
}

/* If type is zero, simply print the task string, otherwise convert the
     type to a string and print task and type.
 */
static void PrintError (task, type)
char *task;
int type;
{
  char *typeString, buf[80];

  if (type == 0) {
    puts (task);
    return;
  }
  
  /* Convert the type to a string if it is recognized.
   */
  if ((typeString = BSAFE2_ErrorString (type)) == (char *)NULL_PTR) {
    sprintf (buf, "Code 0x%04x", type);
    typeString = buf;
  }

  printf ("ERROR: %s while %s\n", typeString, task);  
  fflush (stdout);
}

static int GetCommand (command, maxCommandSize, prompt)
char *command;
unsigned int maxCommandSize;
char *prompt;
{
  unsigned int i;
  
  if (!SILENT_PROMPT) {
    printf ("%s\n", prompt);  
    fflush (stdout);
  }

  fgets (command, maxCommandSize, stdin);

  if (*command == '\n' || *command == '\r')
    return (-1);
  
  /* Replace the line terminator with a '\0'.
   */
  for (i = 0; command[i] != '\0'; i++) {
    if (command[i] == '\n' || command[i] == '\r' ||
        i == (maxCommandSize - 1)) {
      command[i] = '\0';
      return (0);
    }
  }
  return (0);
}

/* Return the error string for the type, or (char *)NULL_PTR if the type
     is not recognized.
 */
static char *BSAFE2_ErrorString (type)
int (type);
{
  switch (type) {
  case BE_ALGORITHM_ALREADY_SET:
    return ("Algorithm object has already been set with algorithm info");
  case BE_ALGORITHM_INFO:
    return ("Invalid algorithm info format");
  case BE_ALGORITHM_NOT_INITIALIZED:
    return ("Algorithm object has not been initialized");
  case BE_ALGORITHM_NOT_SET:
    return ("Algorithm object has not been set with algorithm info");
  case BE_ALGORITHM_OBJ:
    return ("Invalid algorithm object");
  case BE_ALG_OPERATION_UNKNOWN:
    return ("Unknown operation for an algorithm or algorithm info type");
  case BE_ALLOC:
    return ("Insufficient memory");
  case BE_CANCEL:
    return ("Operation was cancelled by the surrender function");
  case BE_DATA:
    return ("Generic data error");
  case BE_EXPONENT_EVEN:
    return ("Invalid even value for public exponent in keypair generation");
  case BE_EXPONENT_LEN:
    return
      ("Invalid exponent length for public exponent in keypair generation");
  case BE_HARDWARE:
    return ("Cryptographic hardware error");
  case BE_INPUT_DATA:
    return ("Invalid format for input data");
  case BE_INPUT_LEN:
    return ("Invalid length for input data");
  case BE_KEY_ALREADY_SET:
    return ("Key object has already been set with key info");
  case BE_KEY_INFO:
    return ("Invalid key info format");
  case BE_KEY_NOT_SET:
    return ("Key object has not been set with key info");
  case BE_KEY_OBJ:
    return ("Invalid key object");
  case BE_KEY_OPERATION_UNKNOWN:
    return ("Unknown operation for a key info type");
  case BE_MEMORY_OBJ:
    return ("Invalid internal memory object");
  case BE_MODULUS_LEN:
    return ("Invalid modulus length in public or private key");
  case BE_NOT_SUPPORTED:
    return ("Unsupported operation requested");
  case BE_OUTPUT_LEN:
    return ("Output data is larger than supplied buffer");
  case BE_OVER_32K:
    return ("Data block exceeds 32,767 bytes");
  case BE_RANDOM_NOT_INITIALIZED:
    return ("Random algorithm has not been initialized");
  case BE_RANDOM_OBJ:
    return ("Invalid algorithm object for the random algorithm");
  case BE_SIGNATURE:
    return ("Invalid signature");
  case BE_WRONG_ALGORITHM_INFO:
    return ("Wrong type of algorithm info");
  case BE_WRONG_KEY_INFO:
    return ("Wrong type of key info");
  default:
    return ((char *)NULL_PTR);
  }
}
