/* Copyright (C) RSA Data Security, Inc. created 1993.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/* define size calculation macros for RSA key and DH parameter generation */

#define MAX_OBJECT_ID_LEN 11
#define MAX_PBE_PARAMETER_LEN (2 + MAX_OBJECT_ID_LEN + 18)
#define MAX_RSA_PARAMETER_LEN 8

#define MAX_RSA_ALGORITHM_ID_LEN \
  (2 + MAX_OBJECT_ID_LEN + MAX_RSA_PARAMETER_LEN)

#define MAX_PASSWORD_PROTECT_DELTA 8
#define PUB_KEY_BYTES(modBits) (23 + MAX_RSA_ALGORITHM_ID_LEN + (modBits / 8))

#define PP_PRIV_KEY_BYTES(modBits) \
  (59 + MAX_RSA_ALGORITHM_ID_LEN + 2 * (modBits / 8) \
   + 5 * ((modBits + 1) / 16) + MAX_PASSWORD_PROTECT_DELTA)

#define DH_PARAMS_BYTES(pBits) \
  (3 + MAX_OBJECT_ID_LEN + 13 + 2 * ((pBits / 8) + 1))

/* define maximum output buffer size for Diffie-Hellman computation,
   RSA signatures and RSA Envelopes
 */

#define MAX_DH_PRIME_BITS 2048
#define MAX_DH_OUTPUT_BYTES (MAX_DH_PRIME_BITS / 8 + 1)
#define MAX_RSA_MODULUS 2048
#define MAX_RSA_OUTPUT_BYTES (MAX_RSA_MODULUS / 8 + 1)
     
/***********************************************************************/
/* symmetric key generation, block and stream cipher                   */
/***********************************************************************/

#define EA_DES_CBC 0
#define EA_RC2_CBC_64 1

#define BLOCK_OUT_BYTES(inputBytes) (((inputBytes) / 8 + 2) * 8)

int  BSL_MakeKeyFromSeed PROTO_LIST
  ((unsigned char *, unsigned int, unsigned char *, unsigned int));

int  BSL_BlockEncryptSetup PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned char *, int));

int  BSL_BlockEncrypt PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *,
    unsigned char *, unsigned int));

int  BSL_BlockDecryptSetup PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned char *, int));

int  BSL_BlockDecrypt PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *,
    unsigned char *, unsigned int));

int  BSL_RC4CipherSetup PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_RC4Cipher PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

/***********************************************************************/
/* Diffie-Hellman parameter generation and 2 phase key negotiation     */
/***********************************************************************/

int  BSL_MakeDHParamsFromSeed PROTO_LIST
  ((unsigned char *, unsigned int *, unsigned int,
    unsigned char *, unsigned int));

int  BSL_DiffieHellman1 PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *,
    unsigned char *, unsigned int, unsigned char *, unsigned int));

int  BSL_DiffieHellman2 PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *,
    unsigned char *, unsigned int));

/***********************************************************************/
/* MD2 or MD5 message digest calculation                               */
/***********************************************************************/

#define DA_MD2 0
#define DA_MD5 1

int  BSL_ComputeDigestInit PROTO_LIST ((B_ALGORITHM_OBJ *, int));

int  BSL_ComputeDigestUpdate PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_ComputeDigestFinal PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *));

/***********************************************************************/
/* rsa key generation, password-based private key protection,          */
/* digital signatures and digital envelopes                            */
/***********************************************************************/

int  BSL_MakeRSAKeyPairFromSeed PROTO_LIST
  ((unsigned char *, unsigned int *, unsigned char *, unsigned int *,
    unsigned char *, unsigned int *, unsigned int, unsigned char *,
    unsigned int, unsigned char *, unsigned int));

int  BSL_MD5WithRSASignInit PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int,
    unsigned char *, unsigned int, unsigned char *, unsigned int));

int  BSL_MD5WithRSASignUpdate PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_MD5WithRSASignFinal PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *));

int  BSL_MD5WithRSAVerifyInit PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_MD5WithRSAVerifyUpdate PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_MD5WithRSAVerifyFinal PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_SealDESWithRSAEnvelopeSetup PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *, unsigned char *,
    unsigned char *, unsigned int, unsigned char *, unsigned int));

int  BSL_OpenDESWithRSAEnvelopeSetup PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int, unsigned char *,
    unsigned char *, unsigned int, unsigned char *, unsigned int,
    unsigned char *, unsigned int));
