/* Copyright (C) RSA Data Security, Inc. created 1994.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

#include <stdio.h>
#include "aglobal.h"
#include "bsafe.h"
#include "fileio.h"

/* Read a file of up to length maxOutputLen bytes, storing it in
     output and returning its length in outputLen.
   Return 0 on success or 1 if error or if user cancels by entering a blank.
 */
int ReadFile (filename, output, outputLen, maxOutputLen) 
char *filename;
unsigned char *output;
unsigned int *outputLen;
unsigned int maxOutputLen;
{
  FILE *file = (FILE *) NULL_PTR;
  unsigned char dummy;
  
  if ((file = fopen (filename, "rb")) == NULL)
      /* could not open file */
    return (BE_DATA);
  
  /* fread () returns the number of items read in.  Expect an end of file
       after the read.
   */
  *outputLen = fread (output, 1, maxOutputLen, file);
  if (*outputLen == maxOutputLen)
    /* Read exactly maxOutputLen bytes, so reading one more will set 
         end of file if there were exactly maxOutputLen bytes in the file.
     */
    fread (&dummy, 1, 1, file);
  
  if (!feof (file)) {
    fclose (file);
    return (BE_DATA);
  }
  
  fclose (file);
  return (0);
}

/* Write the input data of length inputLen to a file.
   Return 0 on success or 1 if error or if user cancels by entering a blank.
 */
int WriteFile (filename, input, inputLen) 
char *filename;
unsigned char *input;
unsigned int inputLen;
{
  FILE *file = (FILE *) NULL_PTR;
  
  if (filename[0] == '-' && filename[1] == '\0')
    /* use stdout */
    file = stdout;
  else {
    if ((file = fopen (filename, "wb")) == NULL)
      /* could not open file */
      return (BE_DATA);
  }
  
  if (fwrite (input, 1, inputLen, file) < inputLen) {
    fclose (file);
    return (BE_DATA);
  }
  else {
    if (file == stdout)
      /* Printing to screen, so print a new line. */
      printf ("\n");
  }

  if (file != stdout)
    fclose (file);
  
  return (0);
}

