/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program will generate a series of random values.  You may want
      to rerun this program using the same seed to prove to yourself that
      you get the same random values.  Please refer to Step 4a:  "The
      Random Seed" for a discussion on generating random values.
 */
  
#include <stdio.h>     
#include <string.h>    
#include "aglobal.h"
#include "bsafe.h"

#define NUMBER_OF_RANDOM_BYTES 64

B_ALGORITHM_METHOD *RANDOM_CHOOSER[] = {
  &AM_SHA_RANDOM,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;
  unsigned char *randomByteBuffer = NULL_PTR;
  unsigned int status;

  do {
    printf ("Generating random bytes \n");
    printf ("======================= \n");

    /*  Step 1:  Create a random algorithm object */
    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    /*  Step 2:  Set the random algorithm object to SHA1 */
    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    /*  Step 3:  Initialize the random algorithm. */
    if ((status = B_RandomInit
         (randomAlgorithm, RANDOM_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update
     */
       /*  Step 4a:  The Random Seed.  First, set aside memory to hold
                     this seed.  Then get the random seed.  The method
                     used here is almost certainly not a method to use
                     when writing a real application.  It is simply for
                     illustrative purposes.
        */
        randomSeedLen = 256;
        randomSeed = T_malloc (randomSeedLen);
        if ((status = (randomSeed == NULL_PTR)) != 0)
          break;

        T_memset (randomSeed, 0, randomSeedLen);

        printf ("Contents of randomSeed before seeding: \n");
        PrintBuf (randomSeed, randomSeedLen);

        puts ("Enter a random seed");
        if ((status =
	     (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
          break;
                
        printf ("Contents of randomSeed after seeding: \n");
        PrintBuf (randomSeed, randomSeedLen);

       /*  Step 4b:  Now we have a random seed and its length.  Pass both
                     into B_RandomUpdate. */
        if ((status = B_RandomUpdate
             (randomAlgorithm, randomSeed, randomSeedLen,
              (A_SURRENDER_CTX *)NULL_PTR)) != 0)
          break;

    /*  Step 5:  Generate.  First, prepare a buffer for receiving the
                 random bytes before calling B_GenerateRandomBytes.
     */
    randomByteBuffer = T_malloc (NUMBER_OF_RANDOM_BYTES);
    if ((status = (randomByteBuffer == NULL_PTR)) != 0)
          break;

    T_memset (randomByteBuffer, 0, NUMBER_OF_RANDOM_BYTES);

    if ((status = B_GenerateRandomBytes
         (randomAlgorithm, randomByteBuffer, NUMBER_OF_RANDOM_BYTES,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("%i bytes of random-generated values: \n",
              NUMBER_OF_RANDOM_BYTES);
    PrintBuf (randomByteBuffer, NUMBER_OF_RANDOM_BYTES);

  } while (0);

  if (status != 0) {
    printf ("Status = %i \n", status);      
    printf ("Generate random bytes failed");
  }

  /*  Step 6:  Remember to destroy all objects, and free all memory
   */
  B_DestroyAlgorithmObject (&randomAlgorithm);
  T_memset (randomSeed, 0, randomSeedLen);
  T_free (randomSeed);
  T_free (randomByteBuffer);

} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
