/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program encrypts and decrypts using RC5 in CBC mode.
 */

#include <stdio.h>   
#include <string.h> 
#include "aglobal.h"
#include "bsafe.h"

#define KEY_SIZE 10    /* number of bytes in the key */
#define BLOCK_SIZE 8   

B_ALGORITHM_METHOD *RC5_CHOOSER[] = {
  &AM_RC5_CBC_ENCRYPT,
  &AM_RC5_CBC_DECRYPT,
  &AM_SHA_RANDOM,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_KEY_OBJ rc5Key = (B_KEY_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ rc5Encrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ rc5Decrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  
  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;

  unsigned char initVector[BLOCK_SIZE];

  A_RC5_CBC_PARAMS rc5Params;

  ITEM rc5KeyItem;

  unsigned char *dataToEncrypt = (unsigned char *)"Encrypt this sentence.";
  unsigned int dataToEncryptLen;

  unsigned char *encryptedData = NULL_PTR;
  unsigned int outputLenUpdate, outputLenFinal, outputLenTotal;
  unsigned int encryptedDataLen;

  unsigned char *decryptedData = NULL_PTR;
  unsigned int decryptedLenUpdate, decryptedLenFinal;
  unsigned int decryptedLenTotal, decryptedDataLen;
  
  unsigned int status = 0;
  
  do {
    printf ("RC5 algorithm:  Encryption phase \n");
    printf ("================================ \n");

    printf ("Input data: %s \n", dataToEncrypt);
    dataToEncryptLen = strlen ((char *)dataToEncrypt) + 1;
    printf ("Input data in hex (%u bytes):\n", dataToEncryptLen);
    PrintBuf ((unsigned char *)dataToEncrypt, dataToEncryptLen);
  
    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */
    printf ("Generating random bytes \n");
    printf ("======================= \n");

    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, RC5_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
         break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
        (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  ========================================================  */

    /*  B_GenerateRandomBytes is called here for 8 bytes.
     */  
    if ((status = B_GenerateRandomBytes
         (randomAlgorithm, (unsigned char *)initVector, BLOCK_SIZE,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    
    printf ("%i bytes of random iv: \n", BLOCK_SIZE);
    PrintBuf (initVector, BLOCK_SIZE);

    /*  Step 1:  Create an algorithm object. */
    if ((status = B_CreateAlgorithmObject (&rc5Encrypter)) != 0)
      break;

    /*  Step 2:  Set the algorithm to a type that does RC5 encryption
        in CBC mode with padding.  */
    
    rc5Params.version = 0x10;
    rc5Params.rounds = 12;
    /* double the word size to get the block size */
    rc5Params.wordSizeInBits = BLOCK_SIZE * 4;
    rc5Params.iv = (unsigned char *)initVector;
    
    if ((status = B_SetAlgorithmInfo
         (rc5Encrypter, AI_RC5_CBCPad, (POINTER)&rc5Params)) != 0)
      break;

      /*  Step 3a:  Create a key object.
        */
       if ((status = B_CreateKeyObject (&rc5Key)) != 0)
         break;

       /*  Step 3b:  Set the key object with a random RC5 key
        */
       rc5KeyItem.data = NULL_PTR;
       rc5KeyItem.len = KEY_SIZE;
       rc5KeyItem.data = T_malloc (rc5KeyItem.len);
       if ((status = (rc5KeyItem.data == NULL_PTR)) != 0)
         break;
          
      if ((status = B_GenerateRandomBytes
           (randomAlgorithm, rc5KeyItem.data, rc5KeyItem.len,
            (A_SURRENDER_CTX *)NULL_PTR)) != 0)
        break;
          
    printf ("%u-byte random RC5 key: \n", rc5KeyItem.len);
    PrintBuf (rc5KeyItem.data, rc5KeyItem.len);

    if ((status = B_SetKeyInfo
         (rc5Key, KI_Item, (POINTER)&rc5KeyItem)) != 0)
      break;

    /*  Let's zeroize the memory and free it up immediately after
          setting the key for security reasons.
     */
    if (rc5KeyItem.data != NULL_PTR) {
      T_memset (rc5KeyItem.data, 0, rc5KeyItem.len);
      T_free (rc5KeyItem.data);
      rc5KeyItem.data = NULL_PTR;
      rc5KeyItem.len = 0;
    }
  
    /*  Step 3:  Init */
    if ((status = B_EncryptInit
         (rc5Encrypter, rc5Key, RC5_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update */
    encryptedDataLen = dataToEncryptLen + BLOCK_SIZE;
    encryptedData = T_malloc (encryptedDataLen);
    if ((status = (encryptedData == NULL_PTR)) != 0)
      break;

    if ((status = B_EncryptUpdate
         (rc5Encrypter, encryptedData, &outputLenUpdate,
          encryptedDataLen, dataToEncrypt, dataToEncryptLen,
	  (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 5:  Final */
    if ((status = B_EncryptFinal
         (rc5Encrypter, encryptedData + outputLenUpdate,
          &outputLenFinal, encryptedDataLen - outputLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    outputLenTotal = outputLenUpdate + outputLenFinal;
    
    printf ("Encrypted data (%u bytes):\n", encryptedDataLen);
    PrintBuf (encryptedData, encryptedDataLen);
    
    printf ("RC5 algorithm:  Decryption phase \n");
    printf ("================================ \n");

    if ((status = B_CreateAlgorithmObject (&rc5Decrypter)) != 0)
      break;

    /*  Use the same AI and parameters as for encryption */
    if ((status = B_SetAlgorithmInfo
         (rc5Decrypter, AI_RC5_CBCPad, (POINTER)&rc5Params)) != 0)
      break;

    /*  Use the same key as for encryption */
    if ((status = B_DecryptInit
         (rc5Decrypter, rc5Key, RC5_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Set the buffer that will take the decrypted data to be the same size
          as the encrypted data's buffer.
     */
    decryptedDataLen = encryptedDataLen;
    decryptedData = T_malloc (decryptedDataLen);
    if ((status = (decryptedData == NULL_PTR)) != 0)
      break;

    if ((status = B_DecryptUpdate
         (rc5Decrypter, decryptedData, &decryptedLenUpdate,
          decryptedDataLen, encryptedData, outputLenTotal,
          (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    if ((status = B_DecryptFinal
         (rc5Decrypter, decryptedData + decryptedLenUpdate,
          &decryptedLenFinal, decryptedDataLen - decryptedLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    decryptedLenTotal = decryptedLenUpdate + decryptedLenFinal;
    printf ("Decrypted data in hex (%u bytes):\n", decryptedLenTotal);
    PrintBuf (decryptedData, decryptedLenTotal);
    
    printf ("Decrypted data: %s \n", decryptedData);
  
  } while (0);

  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Encrypting/Decrypting with RC5 in CBC mode failed");
  }  

  /*  Done with the key and algorithm objects, so destroy them.
   */
  B_DestroyKeyObject (&rc5Key);
  B_DestroyAlgorithmObject (&rc5Encrypter);
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&rc5Decrypter);

  /*  Free up any memory allocated, save it to a file or print it out first
      if you need to save it.
   */
  T_free (randomSeed);
  T_free (encryptedData);
  T_free (decryptedData);

  if (rc5KeyItem.data != NULL_PTR) {
    T_memset (rc5KeyItem.data, 0, rc5KeyItem.len);
    T_free (rc5KeyItem.data);
    rc5KeyItem.data = NULL_PTR;
    rc5KeyItem.len = 0;
  }
  
} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
