/* Copyright (C) RSA Data Security, Inc. created 1992, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

#include <stdio.h>
#include <string.h>
#include "aglobal.h"
#include "bsafe.h"
#include "demochos.h"
#include "bslec.h"

/* If compiling with Borland, define BORLAND_STACK to the stack size,
     such as -DBORLAND_STACK=15000
 */
#ifdef BORLAND_STACK
extern unsigned _stklen = BORLAND_STACK;
#endif

/* If compiling with non-C calling format, define MAIN_CALL to force
     the compiler back to C calling format for main.  Otherwise, this
     sets MAIN_CALL to nothing.
 */
#ifndef MAIN_CALL
#define MAIN_CALL
#endif

int MAIN_CALL main PROTO_LIST ((int, char **));
static int SetOptions PROTO_LIST ((int, char **));
static void DoSignFile PROTO_LIST ((void));
static void DoSealFile PROTO_LIST ((void));
static void DoVerifyFile PROTO_LIST ((void));
static void DoOpenFile PROTO_LIST ((void));
static void DoGenerateKeys PROTO_LIST ((void));
static int GetPublicKey PROTO_LIST ((A_EC_PUBLIC_KEY *, char *));
static int GetPrivateKey PROTO_LIST ((A_EC_PRIVATE_KEY *, char *));
static void PrintMessage PROTO_LIST ((char *));
static void PrintError PROTO_LIST ((char *, int));
static int GetCommand PROTO_LIST ((char *, unsigned int, char *));
static char *BSAFE2_ErrorString PROTO_LIST ((int));

static int SILENT_PROMPT = 0;

/*  EC Parameters  */

/*  version:  0
 *  compress indicator:  CI_NO_COMPRESS
 *  field type:  FT_FP
 *  field element length: 160 bits
 */

static unsigned char FIELD_INFO[20] = {
  0xC2, 0x28, 0x5D, 0xAF, 0xFB, 0x40, 0xD8, 0x8B,
  0x3C, 0xAD, 0xF1, 0x6C, 0x5C, 0x0E, 0xB9, 0x4A,
  0x1B, 0x03, 0x5E, 0x81
};

static unsigned char COEFF_A[20] = {
  0xC2, 0x28, 0x5D, 0xAF, 0xFB, 0x40, 0xD8, 0x8B,
  0x3C, 0xAD, 0xF1, 0x6C, 0x5C, 0x0E, 0xB9, 0x4A,
  0x1B, 0x03, 0x5E, 0x7E
};

static unsigned char COEFF_B[20] = {
  0x8F, 0xF3, 0x57, 0x75, 0x5F, 0xC4, 0x78, 0xA2,
  0x66, 0x11, 0xDF, 0xF0, 0xC3, 0x18, 0xB6, 0x26,
  0xA1, 0x4B, 0xF0, 0x0F
};

static unsigned char BASE[41] = {
  0x04, 0x57, 0xF2, 0xC9, 0xD5, 0xED, 0xC0, 0x8E,
  0xFF, 0x33, 0x35, 0xF2, 0x0C, 0x31, 0x61, 0xAF,
  0x35, 0x37, 0x9B, 0x0A, 0x22, 0x68, 0xE8, 0x80,
  0xF8, 0x4A, 0x28, 0xF4, 0xB3, 0x36, 0xCA, 0x9E,
  0x2E, 0x3B, 0xE2, 0x3A, 0xCC, 0xDF, 0x22, 0x4F,
  0xB1
};

static unsigned char ORDER[20] = {
  0x01, 0x20, 0xFA, 0x97, 0x59, 0x46, 0x54, 0x9B,
  0x99, 0xA7, 0xB4, 0x9E, 0x12, 0x70, 0x03, 0x9F,
  0x8F, 0xDE, 0x2B, 0xBF
};

static unsigned char COFACTOR[1] = {
  0xAC
};

/*  Keypairs generated using the above parameters  */
/*  Note that the public component is one byte larger than twice the
 *  field element length.  The private component is the same length as
 *  the field element.  When changing the EC parameters above, be sure
 *  to adjust the sizes of PUBLIC_KEY3_DATA and PRIVATE_KEY3_DATA
 *  accordingly.  Also, GetPrivateKey() and GetPublicKey() may have to
 *  be modified.
 */

static unsigned char PUBLIC_KEY1_DATA[41] = {
  0x04, 0x56, 0x6B, 0x3D, 0xE5, 0x78, 0xAD, 0xD2,
  0x80, 0x1E, 0x2F, 0x01, 0x82, 0x5F, 0x6C, 0xB3,
  0xD8, 0xA5, 0xE9, 0xF6, 0x21, 0x63, 0x3A, 0x3C,
  0x6D, 0x9F, 0xC3, 0xB8, 0xD5, 0x2A, 0x4D, 0xB4,
  0x05, 0xE0, 0xA3, 0xC1, 0x73, 0xE8, 0x0D, 0x5C,
  0xA5
};

static unsigned char PRIVATE_KEY1_DATA[20] = {
  0x01, 0x12, 0x96, 0x81, 0xC0, 0x29, 0xBD, 0xB2,
  0xEC, 0x31, 0x61, 0x4A, 0xB5, 0x32, 0xF1, 0xA7,
  0x8B, 0x0D, 0x3E, 0xEC
};

static unsigned char PUBLIC_KEY2_DATA[41] = {
  0x04, 0xB2, 0x5B, 0xA7, 0xFC, 0xAE, 0x26, 0xDA,
  0x13, 0x8E, 0xD5, 0x16, 0x91, 0xBC, 0xAB, 0x3C,
  0x2C, 0x4E, 0x0C, 0x31, 0x3B, 0x59, 0x7A, 0x0B,
  0x98, 0x3B, 0xDA, 0x10, 0x09, 0x42, 0xC4, 0xD4,
  0x8B, 0xFC, 0xFF, 0x96, 0x2E, 0x3C, 0x94, 0x68,
  0xFF
};

static unsigned char PRIVATE_KEY2_DATA[20] = {
  0x01, 0x04, 0xB5, 0xA9, 0xA9, 0x22, 0x93, 0x06,
  0xB1, 0x90, 0xF0, 0x2E, 0x5C, 0xF8, 0x98, 0xF9,
  0x26, 0xB2, 0x76, 0x18
};

static unsigned char PUBLIC_KEY3_DATA[41];
static unsigned int PUBLIC_KEY3_LEN = 0;
static unsigned char PRIVATE_KEY3_DATA[20];
static unsigned int PRIVATE_KEY3_LEN = 0;

int MAIN_CALL main (argc, argv)
int argc;
char *argv[];
{
  char command[100];
  int done = 0;

  if (SetOptions (argc, argv))
    return (0);
  
  /* Print version message.
   */
  strcpy (command, "Command-line Demo for BSAFE version ");
  strcpy (command + strlen (command), BSAFE_VERSION);
  PrintMessage (command);

  PrintMessage
    ("NOTE: When saving to a file, a filename of \"-\" will output to the screen.");

  while (!done) {
    PrintMessage ("");
    PrintMessage ("S - Sign a file");
    PrintMessage ("E - Envelope a file");
    PrintMessage ("V - Verify a signed file");
    PrintMessage ("O - Open an enveloped file");
    PrintMessage ("G - Generate a keypair (may take a long time)");
    PrintMessage ("Q - Quit");
    GetCommand (command, sizeof (command), "  Enter choice: ");
    
    switch (*command) {
    case '\0':
    case '#':
      /* entered a blank line or a comment */
      break;
      
    case 's':
    case 'S':
      DoSignFile ();
      break;
      
    case 'e':
    case 'E':
      DoSealFile ();
      break;
      
    case 'v':
    case 'V':
      DoVerifyFile ();
      break;
      
    case 'o':
    case 'O':
      DoOpenFile ();
      break;
      
    case 'g':
    case 'G':
      DoGenerateKeys ();
      break;
      
    case 'Q':
    case 'q':
      done = 1;
      break;
      
    default:
      PrintError ("ERROR: Unrecognized command.  Try again.", 0);
      break;
    }
  }
  
  return (0);
}

/* Set options from command line and return 0 for success, 1 for bad format.
 */
static int SetOptions (argc, argv)
int argc;
char *argv[];
{
  int i, status = 0;
  
  for (i = 1; i < argc; i++) {
    if (argv[i][0] != '-') {
      status = 1;
      break;
    }
    
    if (argv[i][1] == 's')
      SILENT_PROMPT = 1;
    else {
      status = 1;
      break;
    }
  }

  if (status)
    puts ("Usage: bdemoec [-s]\n  -s silent prompts");

  return (status);
}

static void DoSignFile ()
{
  char inputFilename[256], signatureFilename[256], seed[256];
  int status;
  A_EC_PRIVATE_KEY privateKey;

  if (GetCommand (inputFilename, sizeof (inputFilename),
              "  Enter filename of content to sign (blank to cancel): "))
    return;
  if (GetCommand (signatureFilename, sizeof (signatureFilename),
              "  Enter filename to save the signature (blank to cancel): "))
    return;

  if ((*inputFilename == '\0') || (*signatureFilename == '\0'))
    return;
  
  if (GetPrivateKey
      (&privateKey,
       "  Sign as user with private key 1, 2, or 3? (blank to cancel): "))
    return;

  if (GetCommand (seed, sizeof (seed),
       "  Enter seed for EC DSA signing (blank to cancel): "))
    return;  
  
  if ((status = BSL_ECDSAWithSHA1SignFile
       (signatureFilename, inputFilename, &privateKey,
        (unsigned char *)NULL_PTR, 0, (unsigned char *)NULL_PTR, 0,
        (unsigned char *)seed, (unsigned int)strlen(seed))) != 0)
    PrintError ("signing file", status);
  else
    PrintMessage ("File Signed.");
}

static void DoSealFile ()
{
  char encryptedKeyFilename[256], ivFilename[256], outputFilename[256],
    inputFilename[256], seed[256];
  int status;
  A_EC_PUBLIC_KEY publicKey;

  if (GetCommand (inputFilename, sizeof (inputFilename),
       "  Enter filename of content to seal (blank to cancel): "))
    return;
  if (GetCommand (encryptedKeyFilename, sizeof (encryptedKeyFilename),
       "  Enter filename to save the encrypted key (blank to cancel): "))
    return;
  if (GetCommand (ivFilename, sizeof (ivFilename),
       "  Enter filename to save the key's IV (blank to cancel): "))
    return;
  if (GetCommand (outputFilename, sizeof (outputFilename),
       "  Enter filename to save the encrypted content (blank to cancel): "))
    return;
  if (GetCommand (seed, sizeof (seed),
       "  Enter seed to generate random DES key and IV (blank to cancel): "))
    return;

  if (GetPublicKey
      (&publicKey, 
       "  Seal for user with public key 1, 2, or 3? (blank to cancel): "))
    return;
  
  if ((status = BSL_SealFileDESWithECESEnvelope
       (encryptedKeyFilename, ivFilename, outputFilename, inputFilename,
        (unsigned char *)seed, (unsigned int)strlen (seed),
        &publicKey)) != 0) {
    PrintError ("sealing file", status);
    return;
  }
  else
    PrintMessage ("Envelope Sealed.");
}

static void DoVerifyFile ()
{
  char inputFilename[256], signatureFilename[256];
  int status;
  A_EC_PUBLIC_KEY publicKey;
  
  if (GetCommand (inputFilename, sizeof (inputFilename),
              "  Enter name of file to verify (blank to cancel): "))
    return;
  if (GetCommand (signatureFilename, sizeof (signatureFilename),
              "  Enter filename of signature (blank to cancel): "))
    return;

  if ((*inputFilename == '\0') || (*signatureFilename == '\0'))
    return;
  
  if (GetPublicKey
      (&publicKey,
       "  Verify signature from user with public key 1, 2, or 3? (blank to cancel): "))
    return;

  if ((status = BSL_ECDSAWithSHA1VerifyFile
       (signatureFilename, inputFilename, &publicKey)) != 0) {
    PrintError ("verifying file", status);
    return;
  }
  else
    PrintMessage ("Signature verified.");
}

static void DoOpenFile ()
{
  char resultDataFilename[256], encryptedKeyFilename[256], ivFilename[256],
    encryptedDataFilename[256];
  int status;
  A_EC_PRIVATE_KEY privateKey;

  if (GetCommand (encryptedDataFilename, sizeof (encryptedDataFilename),
       "  Enter filename of content to open (blank to cancel): "))
    return;
  if (GetCommand (encryptedKeyFilename, sizeof (encryptedKeyFilename),
       "  Enter filename of the encrypted key (blank to cancel): "))
    return;
  if (GetCommand (ivFilename, sizeof (ivFilename),
       "  Enter filename of the key's IV (blank to cancel): "))
    return;
  if (GetCommand (resultDataFilename, sizeof (resultDataFilename),
       "  Enter filename to save the decrypted content (blank to cancel): "))
    return;

  if (GetPrivateKey
      (&privateKey,
       "  Open for user with private key 1, 2, or 3? (blank to cancel): "))
    return;

  if ((status = BSL_OpenFileDESWithECESEnvelope
       (resultDataFilename, encryptedKeyFilename, ivFilename,
        encryptedDataFilename, &privateKey, 0, 0, 0, 0)) != 0) {
    PrintError ("opening enveloped file", status);
    return;
  }
  else
    PrintMessage ("Envelope opened.");
}

static void DoGenerateKeys ()
{
  int status;
  char seed[256];

  A_EC_PARAMS ecParams;  /*  generate keys for this curve  */
  
  if (PRIVATE_KEY3_LEN != 0) {
    PrintMessage
      ("Public key 3 and private key 3 have already been generated.");
    return;
  }
  
  if ((GetCommand (seed, sizeof (seed),
       "  Enter seed to generate the EC keypair (blank to cancel): ")) != 0)
    return;

  PrintError ("  Generating EC keypair...", 0);

  ecParams.version = 0;
  ecParams.fieldType = FT_FP;
  ecParams.fieldInfo.data = FIELD_INFO;
  ecParams.fieldInfo.len = sizeof(FIELD_INFO);
  ecParams.coeffA.data = COEFF_A;
  ecParams.coeffA.len = sizeof(COEFF_A);
  ecParams.coeffB.data = COEFF_B;
  ecParams.coeffB.len = sizeof(COEFF_B);
  ecParams.base.data = BASE;
  ecParams.base.len = sizeof(BASE);
  ecParams.order.data = ORDER;
  ecParams.order.len = sizeof(ORDER);
  ecParams.cofactor.data = COFACTOR;
  ecParams.cofactor.len = sizeof(COFACTOR);
  ecParams.compressIndicator = CI_NO_COMPRESS;
  ecParams.fieldElementBits = 160;        
  
  if ((status = BSL_MakeECKeyPairFromSeed
       (PUBLIC_KEY3_DATA, &PUBLIC_KEY3_LEN, PRIVATE_KEY3_DATA,
        &PRIVATE_KEY3_LEN, &ecParams, (unsigned char *)NULL_PTR, 0,
        (unsigned char *)seed, (unsigned int)strlen (seed),
        (unsigned char *)NULL_PTR, 0)) != 0) {
    PrintError ("generating keys", status);
    return;
  }

  PrintMessage ("Public key 3 and private key 3 are now ready to use.");
}

/* Use the prompt to ask the user to use public key 1, 2 or 3 and
     point publicKey to the answer.
   Return 0 on success or 1 if user cancels by entering a blank.
 */
static int GetPublicKey (publicKey, prompt)
A_EC_PUBLIC_KEY *publicKey;
char *prompt;
{
  char command[80];
  
  while (1) {
    GetCommand (command, sizeof (command), prompt);

    publicKey->curveParams.version = 0;
    publicKey->curveParams.fieldType = FT_FP;
    publicKey->curveParams.fieldInfo.data = FIELD_INFO;
    publicKey->curveParams.fieldInfo.len = sizeof(FIELD_INFO);
    publicKey->curveParams.coeffA.data = COEFF_A;
    publicKey->curveParams.coeffA.len = sizeof(COEFF_A);
    publicKey->curveParams.coeffB.data = COEFF_B;
    publicKey->curveParams.coeffB.len = sizeof(COEFF_B);
    publicKey->curveParams.base.data = BASE;
    publicKey->curveParams.base.len = sizeof(BASE);
    publicKey->curveParams.order.data = ORDER;
    publicKey->curveParams.order.len = sizeof(ORDER);
    publicKey->curveParams.cofactor.data = COFACTOR;
    publicKey->curveParams.cofactor.len = sizeof(COFACTOR);
    publicKey->curveParams.compressIndicator = CI_NO_COMPRESS;
    publicKey->curveParams.fieldElementBits = 160;        
    
    switch (*command) {
    case '\0':
      return (1);
      
    case '1':
      publicKey->publicKey.data = PUBLIC_KEY1_DATA;
      publicKey->publicKey.len = sizeof(PUBLIC_KEY1_DATA);
      return (0);
      
    case '2':
      publicKey->publicKey.data = PUBLIC_KEY2_DATA;
      publicKey->publicKey.len = sizeof(PUBLIC_KEY2_DATA);
      return (0);
      
    case '3':
      if (PUBLIC_KEY3_LEN == 0) {
        PrintError
          ("ERROR: Public key 3 has not been generated yet.  Try Again.", 0);
        break;
      }
      else {
        publicKey->publicKey.data = PUBLIC_KEY3_DATA;
        publicKey->publicKey.len = PUBLIC_KEY3_LEN;
        return (0);
      }
      
    default:
      if (PUBLIC_KEY3_LEN != 0)
        PrintError ("ERROR: Please enter 1, 2 or 3.  Try again.", 0);
      else
        PrintError ("ERROR: Please enter 1 or 2.  Try again.", 0);
      break;
    }
  }
}

/* Use the prompt to ask the user to use private key 1, 2 or 3 and
     point privateKey to the answer.
   Return 0 on success or 1 if user cancels by entering a blank.
 */
static int GetPrivateKey (privateKey, prompt)
A_EC_PRIVATE_KEY *privateKey;
char *prompt;
{
  char command[80];

  while (1) {
    GetCommand (command, sizeof (command), prompt);

    privateKey->curveParams.version = 0;
    privateKey->curveParams.fieldType = FT_FP;
    privateKey->curveParams.fieldInfo.data = FIELD_INFO;
    privateKey->curveParams.fieldInfo.len = sizeof(FIELD_INFO);
    privateKey->curveParams.coeffA.data = COEFF_A;
    privateKey->curveParams.coeffA.len = sizeof(COEFF_A);
    privateKey->curveParams.coeffB.data = COEFF_B;
    privateKey->curveParams.coeffB.len = sizeof(COEFF_B);
    privateKey->curveParams.base.data = BASE;
    privateKey->curveParams.base.len = sizeof(BASE);
    privateKey->curveParams.order.data = ORDER;
    privateKey->curveParams.order.len = sizeof(ORDER);
    privateKey->curveParams.cofactor.data = COFACTOR;
    privateKey->curveParams.cofactor.len = sizeof(COFACTOR);
    privateKey->curveParams.compressIndicator = CI_NO_COMPRESS;
    privateKey->curveParams.fieldElementBits = 160;    

    switch (*command) {
    case '\0':
      return (1);
      
    case '1':
      privateKey->privateKey.data = PRIVATE_KEY1_DATA;
      privateKey->privateKey.len = sizeof(PRIVATE_KEY1_DATA);
      return (0);
      
    case '2':
      privateKey->privateKey.data = PRIVATE_KEY2_DATA;
      privateKey->privateKey.len = sizeof(PRIVATE_KEY2_DATA);
      return (0);
      
    case '3':
      if (PRIVATE_KEY3_LEN == 0) {
        PrintError
          ("ERROR: Private key 3 has not been generated yet.  Try Again.", 0);
        break;
      }
      else {
        privateKey->privateKey.data = PRIVATE_KEY3_DATA;
        privateKey->privateKey.len = PRIVATE_KEY3_LEN;
        return (0);
      }
      
    default:
      if (PRIVATE_KEY3_LEN != 0)
        PrintError ("ERROR: Please enter 1, 2 or 3.  Try again.", 0);
      else
        PrintError ("ERROR: Please enter 1 or 2.  Try again.", 0);
      break;
    }
  }
}

static void PrintMessage (message)
char *message;
{
  if (!SILENT_PROMPT) {
    puts (message);
    fflush (stdout);
  }
}

/* If type is zero, simply print the task string, otherwise convert the
     type to a string and print task and type.
 */
static void PrintError (task, type)
char *task;
int type;
{
  char *typeString, buf[80];

  if (type == 0) {
    puts (task);
    return;
  }
  
  /* Convert the type to a string if it is recognized.
   */
  if ((typeString = BSAFE2_ErrorString (type)) == (char *)NULL_PTR) {
    sprintf (buf, "Code 0x%04x", type);
    typeString = buf;
  }

  printf ("ERROR: %s while %s\n", typeString, task);  
  fflush (stdout);
}

static int GetCommand (command, maxCommandSize, prompt)
char *command;
unsigned int maxCommandSize;
char *prompt;
{
  unsigned int i;
  
  if (!SILENT_PROMPT) {
    printf ("%s\n", prompt);  
    fflush (stdout);
  }

  fgets (command, maxCommandSize, stdin);

  if (*command == '\n' || *command == '\r')
    return (-1);
  
  /* Replace the line terminator with a '\0'.
   */
  for (i = 0; command[i] != '\0'; i++) {
    if (command[i] == '\n' || command[i] == '\r' ||
        i == (maxCommandSize - 1)) {
      command[i] = '\0';
      return (0);
    }
  }
  return (0);
}

/* Return the error string for the type, or (char *)NULL_PTR if the type
     is not recognized.
 */
static char *BSAFE2_ErrorString (type)
int (type);
{
  switch (type) {
  case BE_ALGORITHM_ALREADY_SET:
    return ("Algorithm object has already been set with algorithm info");
  case BE_ALGORITHM_INFO:
    return ("Invalid algorithm info format");
  case BE_ALGORITHM_NOT_INITIALIZED:
    return ("Algorithm object has not been initialized");
  case BE_ALGORITHM_NOT_SET:
    return ("Algorithm object has not been set with algorithm info");
  case BE_ALGORITHM_OBJ:
    return ("Invalid algorithm object");
  case BE_ALG_OPERATION_UNKNOWN:
    return ("Unknown operation for an algorithm or algorithm info type");
  case BE_ALLOC:
    return ("Insufficient memory");
  case BE_CANCEL:
    return ("Operation was cancelled by the surrender function");
  case BE_DATA:
    return ("Generic data error");
  case BE_EXPONENT_EVEN:
    return ("Invalid even value for public exponent in keypair generation");
  case BE_EXPONENT_LEN:
    return
      ("Invalid exponent length for public exponent in keypair generation");
  case BE_HARDWARE:
    return ("Cryptographic hardware error");
  case BE_INPUT_DATA:
    return ("Invalid format for input data");
  case BE_INPUT_LEN:
    return ("Invalid length for input data");
  case BE_KEY_ALREADY_SET:
    return ("Key object has already been set with key info");
  case BE_KEY_INFO:
    return ("Invalid key info format");
  case BE_KEY_NOT_SET:
    return ("Key object has not been set with key info");
  case BE_KEY_OBJ:
    return ("Invalid key object");
  case BE_KEY_OPERATION_UNKNOWN:
    return ("Unknown operation for a key info type");
  case BE_MEMORY_OBJ:
    return ("Invalid internal memory object");
  case BE_MODULUS_LEN:
    return ("Invalid modulus length in public or private key");
  case BE_NOT_SUPPORTED:
    return ("Unsupported operation requested");
  case BE_OUTPUT_LEN:
    return ("Output data is larger than supplied buffer");
  case BE_OVER_32K:
    return ("Data block exceeds 32,767 bytes");
  case BE_RANDOM_NOT_INITIALIZED:
    return ("Random algorithm has not been initialized");
  case BE_RANDOM_OBJ:
    return ("Invalid algorithm object for the random algorithm");
  case BE_SIGNATURE:
    return ("Invalid signature");
  case BE_WRONG_ALGORITHM_INFO:
    return ("Wrong type of algorithm info");
  case BE_WRONG_KEY_INFO:
    return ("Wrong type of key info");
  default:
    return ((char *)NULL_PTR);
  }
}
