/* Copyright (C) RSA Data Security, Inc. created 1993, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

#ifndef _BSLEC_H_
#define _BSLEC_H_ 1

/***********************************************************************/
/* symmetric key generation, block and stream cipher                   */
/***********************************************************************/

#define EA_DES_CBC 0
#define EA_RC2_CBC_64 1

#define BLOCK_OUT_BYTES(inputBytes) (((inputBytes) / 8 + 2) * 8)

int  BSL_BlockEncrypt PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *,
    unsigned char *, unsigned int));

int  BSL_BlockDecrypt PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *,
    unsigned char *, unsigned int));

/***********************************************************************/
/* ec key generation, password-based private key protection,           */
/* digital signatures and digital envelopes                            */
/***********************************************************************/

/* define size calculation macros for EC key generation */

#define PP_PRIV_KEY_BYTES(fieldElementBits) \
  ((fieldElementBits + 7) / 8)

/* maximum output size for ECDSAWithSHA1 digest */

#define MAX_SIGNATURE_LEN(orderBytes) \
  (6 + 2 * orderBytes)

/* maximum output size for encrypted symmetric key
   We are using ECES to encrypt a DES key.  The maximum length of the
   ciphertext when using ECES is 21 + (2 * field element length in bytes) +
   plaintext length.  */

#define MAX_ENC_SYMKEY_LEN(fieldElementBits) \
  21 + ((fieldElementBits + 7) / 4) + 8

int  BSL_MakeECKeyPairFromSeed PROTO_LIST
  ((unsigned char *, unsigned int *, unsigned char *, unsigned int *,
    A_EC_PARAMS *, unsigned char *, unsigned int *, unsigned char *,
    unsigned int, unsigned char *, unsigned int));

int  BSL_ECDSAWithSHA1SignInit PROTO_LIST
  ((B_ALGORITHM_OBJ *, A_EC_PRIVATE_KEY *,
    unsigned char *, unsigned int, unsigned char *, unsigned int));

int  BSL_ECDSAWithSHA1SignUpdate PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_ECDSAWithSHA1SignFinal PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *, unsigned int,
    unsigned char *, unsigned int));

int  BSL_ECDSAWithSHA1VerifyInit PROTO_LIST
  ((B_ALGORITHM_OBJ *, A_EC_PUBLIC_KEY *));

int  BSL_ECDSAWithSHA1VerifyUpdate PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_ECDSAWithSHA1VerifyFinal PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int));

int  BSL_SealDESWithECESEnvelopeSetup PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int *, unsigned char *,
    unsigned char *, unsigned int, A_EC_PUBLIC_KEY *));

int  BSL_OpenDESWithECESEnvelopeSetup PROTO_LIST
  ((B_ALGORITHM_OBJ *, unsigned char *, unsigned int, unsigned char *,
    A_EC_PRIVATE_KEY *, unsigned char *, unsigned int,
    unsigned char *, unsigned int));

/* BSL file interface */
int BSL_ECDSAWithSHA1SignFile PROTO_LIST
  ((char *, char *, A_EC_PRIVATE_KEY *, unsigned char *, unsigned int,
    unsigned char *, unsigned int, unsigned char *, unsigned int));

int BSL_ECDSAWithSHA1VerifyFile PROTO_LIST
  ((char *, char *, A_EC_PUBLIC_KEY *));

int BSL_SealFileDESWithECESEnvelope PROTO_LIST
  ((char *, char *, char *, char *, unsigned char *, unsigned int,
    A_EC_PUBLIC_KEY *));

int BSL_OpenFileDESWithECESEnvelope PROTO_LIST
  ((char *, char *, char *, char *, A_EC_PRIVATE_KEY *,
    unsigned char *, unsigned int, unsigned char *, unsigned int));

#endif

