/* Copyright (C) RSA Data Security, Inc. created 1997.  This is an
   unpublished work protected as such under copyright law.  This work
   contains proprietary, confidential, and trade secret information of
   RSA Data Security, Inc.  Use, disclosure or reproduction without the
   express written authorization of RSA Data Security, Inc. is
   prohibited.
 */

#ifndef _BHAPI_H_
#define _BHAPI_H_ 1
#include "bsafe.h"

struct B_AlgorithmInfoType;
struct B_KeyInfoType;

typedef struct B_Method {
  struct B_AlgorithmInfoType *algorithmInfoType;
  int encryptFlag;
  struct B_KeyInfoType *keyInfoType;
  POINTER alga;
} B_Method;

typedef int (*HARDWARE_FINAL) (POINTER);

typedef struct {
	B_Method am;
	POINTER globalInfo;
	HARDWARE_FINAL Finalize;
} B_ExtendedMethod;
typedef int (*HARDWARE_INIT) (B_ExtendedMethod *, ITEM *);

typedef struct {
  B_Method *AMPtr;
  POINTER OEM_MethodTable;
  HARDWARE_INIT HardwareInit;
  HARDWARE_FINAL HardwareFinal;
  struct B_KeyInfoType *KI_Replacement;
  unsigned char *oemIdentifier;
} HW_TABLE_ENTRY;
typedef HW_TABLE_ENTRY **HW_TABLE_LIST;

typedef int (*A_ENCRYPT_DECRYPT_ALGA_QUERY) PROTO_LIST
  ((unsigned int *, POINTER, POINTER, B_Method *, B_ExtendedError *));
typedef int (*A_ENCRYPT_DECRYPT_ALGA_INIT) PROTO_LIST
  ((POINTER, POINTER, POINTER, int, B_Method *, B_ExtendedError *,  
    A_SURRENDER_CTX *));
typedef int (*A_ENCRYPT_DECRYPT_ALGA_UPDATE) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int *, unsigned int,
    unsigned char *, unsigned int, A_SURRENDER_CTX *));
typedef int (*A_ENCRYPT_DECRYPT_ALGA_FINAL) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int *, unsigned int,
    A_SURRENDER_CTX *));
typedef int (*A_ENCRYPT_DECRYPT_ALGA_GET_MAX_OUTPUT_LEN) PROTO_LIST
  ((POINTER, unsigned int *, unsigned int));
typedef int (*A_ENCRYPT_DECRYPT_ALGA_GET_BLOCK_LEN) PROTO_LIST
  ((POINTER, unsigned int *));
typedef int (*A_ENCRYPT_DECRYPT_ALGA_CTX_DESTRUCTOR) PROTO_LIST
  ((POINTER));

typedef struct {
  A_ENCRYPT_DECRYPT_ALGA_QUERY Query;
  A_ENCRYPT_DECRYPT_ALGA_INIT Init;
  A_ENCRYPT_DECRYPT_ALGA_UPDATE Update;
  A_ENCRYPT_DECRYPT_ALGA_FINAL Final;
  A_ENCRYPT_DECRYPT_ALGA_GET_MAX_OUTPUT_LEN GetMaxOutputLen;
  A_ENCRYPT_DECRYPT_ALGA_GET_BLOCK_LEN GetBlockLen;
  A_ENCRYPT_DECRYPT_ALGA_CTX_DESTRUCTOR SetContextDestructor;
} A_ENCRYPT_DECRYPT_ALGA;


typedef int (*A_RANDOM_ALGA_QUERY) PROTO_LIST
  ((unsigned int *, POINTER,  B_Method *, B_ExtendedError *));
typedef int (*A_RANDOM_ALGA_INIT) PROTO_LIST
  ((POINTER, POINTER, B_Method *, B_ExtendedError *, A_SURRENDER_CTX *));
typedef int (*A_RANDOM_ALGA_UPDATE) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int, A_SURRENDER_CTX *));
typedef int (*A_RANDOM_ALGA_GENERATE) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int, A_SURRENDER_CTX *));
typedef int (*A_RANDOM_ALGA_CTX_DESTRUCTOR) PROTO_LIST ((POINTER));
typedef struct {
  A_RANDOM_ALGA_QUERY Query;
  A_RANDOM_ALGA_INIT Init;
  A_RANDOM_ALGA_UPDATE Update;
  A_RANDOM_ALGA_GENERATE Generate;
  A_RANDOM_ALGA_CTX_DESTRUCTOR SetContextDestructor;
} A_RANDOM_ALGA;


typedef int (*A_SYM_KEY_GEN_ALGA_QUERY) PROTO_LIST
  ((unsigned int *, unsigned int *, POINTER,  B_Method *, B_ExtendedError *));
typedef int (*A_SYM_KEY_GEN_ALGA_INIT) PROTO_LIST
  ((POINTER, POINTER, B_Method *, B_ExtendedError *, A_SURRENDER_CTX *));
typedef int (*A_SYM_KEY_GEN_ALGA_GENERATE) PROTO_LIST
  ((POINTER, POINTER *, unsigned char *, A_SURRENDER_CTX *));
typedef int (*A_SYM_KEY_GEN_ALGA_CTX_DESTR) PROTO_LIST
  ((POINTER));

typedef struct {
  A_SYM_KEY_GEN_ALGA_QUERY Query;
  A_SYM_KEY_GEN_ALGA_INIT Init;
  A_SYM_KEY_GEN_ALGA_GENERATE Generate;
  A_SYM_KEY_GEN_ALGA_CTX_DESTR SetContextDestructor;
} A_SYM_KEY_GEN_ALGA;


typedef int (*A_GENERATE_ALGA_QUERY) PROTO_LIST
  ((unsigned int *, unsigned int *, unsigned int *, struct B_KeyInfoType **,
    struct B_KeyInfoType **, POINTER,  B_Method *, B_ExtendedError *));
typedef int (*A_GENERATE_ALGA_INIT) PROTO_LIST
  ((POINTER, POINTER, POINTER, B_Method *, B_ExtendedError *, 
    A_SURRENDER_CTX *));
typedef int (*A_GENERATE_ALGA_GENERATE) PROTO_LIST
  ((POINTER, POINTER *, unsigned char *, A_SURRENDER_CTX *));
typedef int (*A_GENERATE_ALGA_CTX_DESTRUCTOR) PROTO_LIST
  ((POINTER));

typedef struct {
  A_GENERATE_ALGA_QUERY Query;
  A_GENERATE_ALGA_INIT Init;
  A_GENERATE_ALGA_GENERATE Generate;
  A_GENERATE_ALGA_CTX_DESTRUCTOR SetContextDestructor;
} A_GENERATE_ALGA;


typedef int (*A_DSA_ALGA_QUERY) PROTO_LIST
  ((unsigned int *, POINTER, POINTER, B_Method *, B_ExtendedError *));
typedef int (*A_DSA_ALGA_INIT) PROTO_LIST
  ((POINTER, POINTER, POINTER, B_Method *, B_ExtendedError *, 
    A_SURRENDER_CTX *));
typedef int (*A_DSA_ALGA_SIGN) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int *, unsigned int, unsigned char *,
    unsigned int, unsigned char *, A_SURRENDER_CTX *));
typedef int (*A_DSA_ALGA_VERIFY) PROTO_LIST
  ((POINTER, int *, unsigned char *, unsigned int, unsigned char *,
    unsigned int, A_SURRENDER_CTX *));
typedef int (*A_DSA_ALGA_GET_BLOCKLEN)
  PROTO_LIST ((POINTER, unsigned int *));
typedef int (*A_DSA_ALGA_GET_RANDOM_BLOCKLEN) PROTO_LIST
  ((POINTER, unsigned int *));
typedef int (*A_DSA_ALGA_CTX_DESTRUCTOR) PROTO_LIST ((POINTER));

typedef struct {
  A_DSA_ALGA_QUERY Query;
  A_DSA_ALGA_INIT Init;
  A_DSA_ALGA_SIGN Sign;
  A_DSA_ALGA_VERIFY Verify;
  A_DSA_ALGA_GET_BLOCKLEN GetBlockLen;
  A_DSA_ALGA_GET_RANDOM_BLOCKLEN GetRandomBlockLen;
  A_DSA_ALGA_CTX_DESTRUCTOR SetContextDestructor;  
} A_DSA_ALGA;

typedef int (*A_BLOCK_CRYPTOR_QUERY) PROTO_LIST
  ((unsigned int *, POINTER, POINTER, B_Method *, B_ExtendedError *));
typedef int (*A_BLOCK_CRYPTOR_INIT) PROTO_LIST
  ((POINTER, POINTER, POINTER, int, B_Method *, B_ExtendedError *,  
    A_SURRENDER_CTX *));
typedef int (*A_BLOCK_CRYPTOR) PROTO_LIST
  ((POINTER, unsigned char *, unsigned char *));
typedef int (*A_BLOCK_CRYPTOR_GET_BLOCK_LEN) PROTO_LIST
  ((POINTER, unsigned int *));
typedef int (*A_BLOCK_CRYPTOR_DESTRUCTOR) PROTO_LIST ((POINTER));

typedef struct {
  A_BLOCK_CRYPTOR_QUERY Query; /* used to test params and size algae context */
  A_BLOCK_CRYPTOR_INIT Init;    /* Takes in key and params and inits context */
  A_BLOCK_CRYPTOR Encrypt;   /* Both Encrypt and Decrypt are active for TDEA */
  A_BLOCK_CRYPTOR Decrypt;   /* Otherwise one is a stump that returns        */
			     /* AE_NOT_SUPPORTED                             */
  A_BLOCK_CRYPTOR_GET_BLOCK_LEN GetBlockLen;
  A_BLOCK_CRYPTOR_DESTRUCTOR SetContextDestructor;
  unsigned int cryptTypeIDNum;    /* assembly code uses it to avoid function */
			          /* call overhead                           */
} A_BLOCK_CRYPTOR_ALGA;


typedef int (*A_KEY_AGREE_ALGA_QUERY) PROTO_LIST
  ((unsigned int *, unsigned int *, unsigned int *, POINTER, POINTER,
    B_Method *, B_ExtendedError *));
typedef int (*A_KEY_AGREE_ALGA_INIT) PROTO_LIST
  ((POINTER, POINTER, POINTER, POINTER, B_Method *, B_ExtendedError *,
    A_SURRENDER_CTX *));
typedef int (*A_KEY_AGREE_ALGA_PHASE1) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int *, unsigned int, unsigned char *,
    A_SURRENDER_CTX *));
typedef int (*A_KEY_AGREE_ALGA_PHASE2) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int *, unsigned int, unsigned char *,
    unsigned int, A_SURRENDER_CTX *));
typedef int (*A_KEY_AGREE_ALGA_GET_MAX_OUTPUT_LEN) PROTO_LIST
  ((POINTER, unsigned int *));
typedef int (*A_KEY_AGREE_ALGA_CTX_DESTRUCTOR) PROTO_LIST
  ((POINTER));

typedef struct {
  A_KEY_AGREE_ALGA_QUERY Query;
  A_KEY_AGREE_ALGA_INIT Init;
  A_KEY_AGREE_ALGA_PHASE1 Phase1;
  A_KEY_AGREE_ALGA_PHASE2 Phase2;
  A_KEY_AGREE_ALGA_GET_MAX_OUTPUT_LEN GetMaxOutputLen;
  A_KEY_AGREE_ALGA_CTX_DESTRUCTOR SetContextDestructor;
} A_KEY_AGREE_ALGA;


typedef int (*A_DIGEST_ALGA_QUERY) PROTO_LIST
   ((unsigned int *, POINTER, B_Method *, B_ExtendedError *));
typedef int (*A_DIGEST_ALGA_INIT) PROTO_LIST
  ((POINTER, POINTER, B_Method *, B_ExtendedError *, A_SURRENDER_CTX *));
typedef int (*A_DIGEST_ALGA_UPDATE) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int, A_SURRENDER_CTX *));
typedef int (*A_DIGEST_ALGA_FINAL) PROTO_LIST
  ((POINTER, unsigned char *, unsigned int *, unsigned int,
    A_SURRENDER_CTX *));
typedef int (*A_DIGEST_ALGA_GET_MAX_OUTPUT_LEN) PROTO_LIST
   ((POINTER, unsigned int *));

typedef struct {
  A_DIGEST_ALGA_QUERY Query;
  A_DIGEST_ALGA_INIT Init;
  A_DIGEST_ALGA_UPDATE Update;
  A_DIGEST_ALGA_FINAL Final;
  A_DIGEST_ALGA_GET_MAX_OUTPUT_LEN GetMaxOutputLen;
} A_DIGEST_ALGA;




#if RSA_BSAFE_DESTINATION == RSA_DOMESTIC_VERSION
/* Domestic, unrestricted key sizes */
#define BSAFE_MAX_RSA_KEY_SIZE_BITS     2048
#define BSAFE_MIN_RSA_KEY_SIZE_BITS     256

#define BSAFE_MAX_RC2_KEY_SIZE_BITS     1024
#define BSAFE_MIN_RC2_KEY_SIZE_BITS     1

#define BSAFE_MAX_RC4_KEY_SIZE_BITS     2048
#define BSAFE_MIN_RC4_KEY_SIZE_BITS     8

#define BSAFE_MAX_ECFP_KEY_SIZE_BITS    384
#define BSAFE_MIN_ECFP_KEY_SIZE_BITS    64

#define BSAFE_MAX_ECF2_KEY_SIZE_BITS    384
#define BSAFE_MIN_ECF2_KEY_SIZE_BITS    64
#endif /* RSA_DOMESTIC_VERSION */

#if RSA_BSAFE_DESTINATION == RSA_EXPORT_VERSION
/* international, restricted key sizes */
#define BSAFE_MAX_RSA_KEY_SIZE_BITS     512
#define BSAFE_MIN_RSA_KEY_SIZE_BITS     256

#define BSAFE_MAX_RC2_KEY_SIZE_BITS     40
#define BSAFE_MIN_RC2_KEY_SIZE_BITS     1

#define BSAFE_MAX_RC4_KEY_SIZE_BITS     40
#define BSAFE_MIN_RC4_KEY_SIZE_BITS     8

#define BSAFE_MAX_ECFP_KEY_SIZE_BITS    128
#define BSAFE_MIN_ECFP_KEY_SIZE_BITS    64

#define BSAFE_MAX_ECF2_KEY_SIZE_BITS    128
#define BSAFE_MIN_ECF2_KEY_SIZE_BITS    64

#endif  /* RSA_EXPORT_VERSION */

#define BSAFE_MAX_DH_KEY_SIZE_BITS      BSAFE_MAX_RSA_KEY_SIZE_BITS
#define BSAFE_MIN_DH_KEY_SIZE_BITS      BSAFE_MIN_RSA_KEY_SIZE_BITS

#define BSAFE_MAX_DSA_KEY_SIZE_BITS     BSAFE_MAX_RSA_KEY_SIZE_BITS
#define BSAFE_MIN_DSA_KEY_SIZE_BITS     512
#endif /* _BHAPI_H_ */
