/* Copyright (C) RSA Data Security, Inc. created 1995.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program takes the Introductory Example one step further, by
      providing the BER-encoded algorithm identifier. 
 */

#include <stdio.h>         
#include <string.h>     
#include "aglobal.h"
#include "bsafe.h"
#include "demochos.h" 

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_KEY_OBJ rc4Key = (B_KEY_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ encryptionObject = (B_ALGORITHM_OBJ)NULL_PTR;

  /*  As with the IV, the normal application would likely use a random
        number generator to produce eight bytes for the key.  */
  static unsigned char rc4KeyData[10] = {
    0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12
  };
  unsigned int rc4KeyDataLen = 10;
  ITEM rc4KeyItem;

  static char *dataToEncrypt = "Encrypt this sentence.";
  unsigned char *encryptedData = NULL_PTR;
  unsigned int dataToEncryptLen, outputBufferSize;
  unsigned int outputLenUpdate, outputLenFinal;
  unsigned int status;
  
  ITEM *getInfoBER;
  ITEM infoBER;

  do {
    /*  Find out how many bytes to encrypt. */
    dataToEncryptLen = (unsigned int)strlen (dataToEncrypt);
  
    printf ("The Introductory Example w/BER retrieval\n");
    printf ("========================================\n");

    printf ("Input data: %s \n", dataToEncrypt);
    printf ("Input data in hex: \n");
    PrintBuf ((unsigned char *)dataToEncrypt, dataToEncryptLen);

    infoBER.data = NULL_PTR;
  
    /*  Allocate a buffer that is at least 8 bytes longer than the
          length of the input buffer to hold padding.  */
    encryptedData = T_malloc
        (outputBufferSize = dataToEncryptLen);
    if ((status = (encryptedData == NULL_PTR)) != 0)
      break;
    
    /*  Create a key object. */
    if ((status = B_CreateKeyObject (&rc4Key)) != 0)
      break;

    /*  Set the key to be a RC4 key and set it to the rc4KeyData declared
          above. */
    rc4KeyItem.data = rc4KeyData;
    rc4KeyItem.len = rc4KeyDataLen;
    if ((status = B_SetKeyInfo
         (rc4Key, KI_Item, (POINTER)&rc4KeyItem)) != 0)
      break;

    /*  Create an algorithm object. */
    if ((status = B_CreateAlgorithmObject
         (&encryptionObject)) != 0)
      break;

    /*  Set the algorithm to a type that does RC4 encryption. */
    if ((status = B_SetAlgorithmInfo
         (encryptionObject, AI_RC4, NULL_PTR)) != 0)
      break;

    /*  Getting the BER-encoded algorithm identifier */
    if ((status = B_GetAlgorithmInfo
         ((POINTER *)&getInfoBER, encryptionObject,
          AI_RC4_BER)) != 0)
      break;

    infoBER.len = getInfoBER->len;
    infoBER.data = T_malloc (infoBER.len);
    if ((status = (infoBER.data == NULL_PTR)) != 0)
      break;
    T_memcpy (infoBER.data, getInfoBER->data, infoBER.len);

    printf ("The BER-encoded algorithm identifier:\n");
    PrintBuf (infoBER.data, infoBER.len);
    
    /*  Ready to encrypt. Use B_EncryptInit, B_EncryptUpdate
        and B_EncryptFinal.
     */
    if ((status = B_EncryptInit
         (encryptionObject, rc4Key, DEMO_ALGORITHM_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    if ((status = B_EncryptUpdate
         (encryptionObject, encryptedData, &outputLenUpdate,
          outputBufferSize, (unsigned char *)dataToEncrypt,
          dataToEncryptLen, (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("outputLenUpdate = %i \n", outputLenUpdate);

    if ((status = B_EncryptFinal
         (encryptionObject, encryptedData + outputLenUpdate,
          &outputLenFinal, outputBufferSize - outputLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("outputLenFinal = %i \n", outputLenFinal);
    printf ("Total length of encrypted data = %i \n",
            (outputLenUpdate + outputLenFinal));

    printf ("Encrypted data: \n");
    PrintBuf (encryptedData, (outputLenUpdate + outputLenFinal));

  } while (0);
    
  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("The Introductory Example w/BER retrieval failed");
  }

  /*  Done with the key and algorithm objects, so destroy them.
   */
  B_DestroyKeyObject (&rc4Key);
  B_DestroyAlgorithmObject (&encryptionObject);

  /*  Free up any memory allocated and zeroize any sensitive memory.
        Save it to a file or print it out first if you need to save it.
   */

  T_free (encryptedData);

  if (infoBER.data != NULL_PTR) {
    T_memset (infoBER.data, 0, infoBER.len);
    T_free (infoBER.data);
    infoBER.data = NULL_PTR;
    infoBER.len = 0;
  }
  
  T_memset (rc4KeyData, 0, 10);
  
} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
