/* Copyright (C) RSA Data Security, Inc. created 1995, 1998.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program encrypts and decrypts using AI_RC4.
 */

#include <stdio.h>   
#include <string.h> 
#include "aglobal.h"
#include "bsafe.h"

#define KEY_SIZE 10    /* key size in bytes */

B_ALGORITHM_METHOD *RC4_CHOOSER[] = {
  &AM_RC4_ENCRYPT,
  &AM_RC4_DECRYPT,
  &AM_SHA_RANDOM,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_KEY_OBJ rc4Key = (B_KEY_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ rc4Encrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ rc4Decrypter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  
  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;
  ITEM rc4KeyItem;

  static char *dataToEncrypt = "Encrypt this sentence.";
  unsigned int dataToEncryptLen;

  unsigned char *encryptedData = NULL_PTR;
  unsigned int outputLenUpdate, outputLenFinal, outputLenTotal;
  unsigned int encryptedDataLen;

  unsigned char *decryptedData = NULL_PTR;
  unsigned int decryptedLenUpdate, decryptedLenFinal;
  unsigned int decryptedLenTotal, decryptedDataLen;
  
  unsigned int status;
  
  do {
    printf ("RC4 algorithm:  Encryption phase \n");
    printf ("================================ \n");

    printf ("Input data: %s \n", dataToEncrypt);
    dataToEncryptLen = strlen (dataToEncrypt) + 1;
    printf ("Input data in hex (%u bytes):\n", dataToEncryptLen);
    PrintBuf ((unsigned char *)dataToEncrypt, dataToEncryptLen);
  
    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */
    printf ("Generating random bytes \n");
    printf ("======================= \n");

    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_SHA1Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, RC4_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
         break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  ========================================================  */
    
    /*  Step 1:  Create an algorithm object. */
    if ((status = B_CreateAlgorithmObject (&rc4Encrypter)) != 0)
      break;

    /*  Step 2:  Set the algorithm to a type that does rc4 encryption.
                 AI_RC4 will do. */
    if ((status = B_SetAlgorithmInfo
         (rc4Encrypter, AI_RC4, NULL_PTR)) != 0)
      break;
    
       /*  Step 3a:  Create a key object.
        */
       if ((status = B_CreateKeyObject (&rc4Key)) != 0)
         break;
        
       /*  Step 3b:  Set the key object with a random 10-byte key
        */
       rc4KeyItem.data = NULL_PTR;
       rc4KeyItem.len = KEY_SIZE;
       rc4KeyItem.data = T_malloc (rc4KeyItem.len);
       if ((status = (rc4KeyItem.data == NULL_PTR)) != 0)
         break;
          
      if ((status = B_GenerateRandomBytes
           (randomAlgorithm, rc4KeyItem.data, rc4KeyItem.len,
            (A_SURRENDER_CTX *)NULL_PTR)) != 0)
        break;

    printf ("%u-byte random RC4 key: \n", rc4KeyItem.len);
    PrintBuf (rc4KeyItem.data, rc4KeyItem.len);

    if ((status = B_SetKeyInfo
         (rc4Key, KI_Item, (POINTER)&rc4KeyItem)) != 0)
      break;

    /*  Let's zeroize the memory and free it up immediately after
          setting the key for security reasons.
     */
    if (rc4KeyItem.data != NULL_PTR) {
      T_memset (rc4KeyItem.data, 0, rc4KeyItem.len);
      T_free (rc4KeyItem.data);
      rc4KeyItem.data = NULL_PTR;
      rc4KeyItem.len = 0;
    }
  
    /*  Step 3:  Init */
    if ((status = B_EncryptInit
         (rc4Encrypter, rc4Key, RC4_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update */
    encryptedDataLen = dataToEncryptLen;
    encryptedData = T_malloc (encryptedDataLen);
    if ((status = (encryptedData == NULL_PTR)) != 0)
      break;

    if ((status = B_EncryptUpdate
         (rc4Encrypter, encryptedData, &outputLenUpdate,
          encryptedDataLen, (unsigned char *)dataToEncrypt,
          dataToEncryptLen, (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 5:  Final */
    if ((status = B_EncryptFinal
         (rc4Encrypter, encryptedData + outputLenUpdate,
          &outputLenFinal, encryptedDataLen - outputLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR, (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    outputLenTotal = outputLenUpdate + outputLenFinal;
    printf ("Encrypted data (%u bytes):\n", encryptedDataLen);    
    PrintBuf (encryptedData, encryptedDataLen);
    
    printf ("RC4 algorithm:  Decryption phase \n");
    printf ("================================ \n");

    if ((status = B_CreateAlgorithmObject (&rc4Decrypter)) != 0)
      break;

    /*  Use the same AI and initialization vector as for encryption */
    if ((status = B_SetAlgorithmInfo
         (rc4Decrypter, AI_RC4, NULL_PTR)) != 0)
      break;

    /*  Use the same key as for encryption */
    if ((status = B_DecryptInit
         (rc4Decrypter, rc4Key, RC4_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Set the buffer that will take the decrypted data to be the same size
          as the encrypted data's buffer.
     */
    decryptedDataLen = encryptedDataLen;
    decryptedData = T_malloc (decryptedDataLen);
    if ((status = (decryptedData == NULL_PTR)) != 0)
      break;

    if ((status = B_DecryptUpdate
         (rc4Decrypter, decryptedData, &decryptedLenUpdate,
          decryptedDataLen, encryptedData, decryptedDataLen,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    
    if ((status = B_DecryptFinal
         (rc4Decrypter, decryptedData + decryptedLenUpdate,
          &decryptedLenFinal, decryptedDataLen - decryptedLenUpdate,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    decryptedLenTotal = decryptedLenUpdate + decryptedLenFinal;
    printf ("Decrypted data in hex (%u bytes):\n", decryptedLenTotal);
    PrintBuf (decryptedData, decryptedLenTotal);
    
    printf ("Decrypted data: %s \n", decryptedData);
  
  } while (0);

  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Encrypting/Decrypting with AI_RC4 failed");
  }  

  /*  Done with the key and algorithm objects, so destroy them.
   */
  B_DestroyKeyObject (&rc4Key);
  B_DestroyAlgorithmObject (&rc4Encrypter);
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&rc4Decrypter);

  /*  Free up any memory allocated, save it to a file or print it out first
      if you need to save it.
   */
  if (randomSeed != NULL_PTR) {
    T_memset (randomSeed, 0, randomSeedLen);
    T_free (randomSeed);
    randomSeed = NULL_PTR;
  }
  
  if (encryptedData != NULL_PTR) {
    T_memset (encryptedData, 0, encryptedDataLen);
    T_free (encryptedData);
    encryptedData = NULL_PTR;
  }

  if (decryptedData != NULL_PTR) {
    T_memset (decryptedData, 0, decryptedDataLen);
    T_free (decryptedData);
    decryptedData = NULL_PTR;
  }

  if (rc4KeyItem.data != NULL_PTR) {
    T_memset (rc4KeyItem.data, 0, rc4KeyItem.len);
    T_free (rc4KeyItem.data);
    rc4KeyItem.data = NULL_PTR;
    rc4KeyItem.len = 0;
  }
  
} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
