/* Copyright (C) RSA Data Security, Inc. created 1995.

   This file is used to demonstrate how to interface to an
   RSA Data Security, Inc. licensed development product.

   You have a royalty-free right to use, modify, reproduce and
   distribute this demonstration file (including any modified
   version), provided that you agree that RSA Data Security,
   Inc. has no warranty, implied or otherwise, or liability
   for this demonstration file or any modified version.
 */

/*  This program will create a number of shares of the encrypted information.
      Then recover the information by collecting a declared number of the
      shares.
 */
     
#include <stdio.h>    
#include <string.h>   
#include <time.h>
#include "aglobal.h"
#include "bsafe.h"

B_ALGORITHM_METHOD *SS_SAMPLE_CHOOSER[] = {
  &AM_MD5_RANDOM,
  (B_ALGORITHM_METHOD *)NULL_PTR
};

#define SECRET_SIZE   16
#define TOTAL_SHARES   4

void PrintBuf PROTO_LIST ((unsigned char *, unsigned int));

void main()
{
  B_ALGORITHM_OBJ secretSplitter = (B_ALGORITHM_OBJ)NULL_PTR;
  B_SECRET_SHARING_PARAMS secretSharingParams;

  B_ALGORITHM_OBJ randomAlgorithm = (B_ALGORITHM_OBJ)NULL_PTR;
  POINTER randomSeed = NULL_PTR;
  unsigned int randomSeedLen;

  static unsigned char secretKey[SECRET_SIZE] = {
    0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
    0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10
  };
  unsigned char *secretShare[TOTAL_SHARES];
  unsigned int secretShareLen[TOTAL_SHARES];
  int count;      

  B_ALGORITHM_OBJ secretReconstructer = (B_ALGORITHM_OBJ)NULL_PTR;
  unsigned char getSecret[SECRET_SIZE];
  unsigned int getSecretLen;
  unsigned int outputLenUpdate, outputLenFinal;
  
  unsigned int status;
  
  do {
    for (count = 0; count < TOTAL_SHARES; ++count)
      secretShare[count] = NULL_PTR;

    printf ("Secret Sharing Algorithm \n");
    printf ("======================== \n");

    /*  ========================================================  */
    /*  The following code was pulled from genbytes.c, steps 1-4.
     */
    if ((status = B_CreateAlgorithmObject (&randomAlgorithm)) != 0)
      break;

    if ((status = B_SetAlgorithmInfo
         (randomAlgorithm, AI_MD5Random, NULL_PTR)) != 0)
      break;

    if ((status = B_RandomInit
         (randomAlgorithm, SS_SAMPLE_CHOOSER,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    randomSeedLen = 256;
    randomSeed = T_malloc (randomSeedLen);
    if ((status = (randomSeed == NULL_PTR)) != 0)
         break;

    T_memset (randomSeed, 0, randomSeedLen);

    puts ("Enter a random seed");
    if ((status =
	 (NULL_PTR == (unsigned char *)gets ((char *)randomSeed))) != 0)
      break;

    if ((status = B_RandomUpdate
         (randomAlgorithm, randomSeed, randomSeedLen,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;
    /*  ========================================================  */

    printf ("\nHere's your secret key to be split into %i shares:\n",
            TOTAL_SHARES);
    PrintBuf (secretKey, SECRET_SIZE);

    printf ("   Generating the shares \n");
    printf ("   ===================== \n");

    /*  Step 1:  Create algorithm object */
    if ((status = B_CreateAlgorithmObject (&secretSplitter)) != 0)
      break;

    /*  Step 2:  Set algorithm object to AI_BSSecretSharing */
    secretSharingParams.threshold = 2;
    if ((status = B_SetAlgorithmInfo
         (secretSplitter, AI_BSSecretSharing,
          (POINTER)&secretSharingParams)) != 0)
      break;

    /*  Step 3:  Init */
    if ((status = B_EncryptInit
         (secretSplitter, (B_KEY_OBJ)NULL_PTR,
          (B_ALGORITHM_CHOOSER)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update
     */
    for (count = 0; count < TOTAL_SHARES; ++count) {
      secretShare[count] = T_malloc (SECRET_SIZE + 1);
      if ((status = (secretShare[count] == NULL_PTR)) != 0)
        break;

      if ((status = B_EncryptUpdate
           (secretSplitter, secretShare[count],
            &(secretShareLen[count]), SECRET_SIZE + 1,
            secretKey, SECRET_SIZE, randomAlgorithm,
            (A_SURRENDER_CTX *)NULL_PTR)) != 0)
        break;

      printf ("Secret share #%i = \n", (count+1));
      PrintBuf (secretShare[count], secretShareLen[count]);
    } /*  end for  */
    if (status != 0)
      break;

    /*  Step 5:  Final */
    if ((status = B_EncryptFinal
         (secretSplitter, NULL_PTR, &outputLenFinal, 0,             
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    printf ("   Reconstructing the secret \n");
    printf ("   ========================= \n");

    /*  Step 1:  Create algorithm object */
    if ((status = B_CreateAlgorithmObject (&secretReconstructer)) != 0)
      break;

    /*  Step 2:  Set algorithm object to AI_BSSecretSharing */
    secretSharingParams.threshold = 2;
    if ((status = B_SetAlgorithmInfo
         (secretReconstructer, AI_BSSecretSharing,
          (POINTER)&secretSharingParams)) != 0)
      break;

    /*  Step 3:  Init */
    if ((status = B_DecryptInit
         (secretReconstructer, (B_KEY_OBJ)NULL_PTR,
          (B_ALGORITHM_CHOOSER)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;

    /*  Step 4:  Update
     */
    for (count = 0; count < (int)secretSharingParams.threshold; ++count) {
      if ((status = B_DecryptUpdate
           (secretReconstructer, NULL_PTR, &outputLenUpdate,
            0, secretShare[count], secretShareLen[count],
            (B_ALGORITHM_OBJ)NULL_PTR,
            (A_SURRENDER_CTX *)NULL_PTR)) != 0)
        break;
    } /*  end for  */
    if (status != 0)
      break;

    /*  Step 5:  Final */
    if ((status = B_DecryptFinal
         (secretReconstructer, getSecret, &getSecretLen, SECRET_SIZE,
          (B_ALGORITHM_OBJ)NULL_PTR,
          (A_SURRENDER_CTX *)NULL_PTR)) != 0)
      break;


    printf ("Your secret key has been reconstructed using %i shares.\n",
             secretSharingParams.threshold);
    printf ("This is your secret key:\n");
    PrintBuf (getSecret, getSecretLen);

  } while (0);

  if (status != 0) {
    printf ("Status = %i \n", status);
    printf ("Secret Sharing failed");
  }  
  
  /*  Step 6:  Destroy
   */
  B_DestroyAlgorithmObject (&secretSplitter);           
  B_DestroyAlgorithmObject (&randomAlgorithm);
  B_DestroyAlgorithmObject (&secretReconstructer);

  /*  Free up any memory allocated
   */
  T_free (randomSeed);
  for (count = 0; count < TOTAL_SHARES; ++count)
    T_free (secretShare[count]);

} /*  end main  */


/*  This procedure will print out what's in the buffer.
 */
void PrintBuf (buffer, bufferLen)
unsigned char *buffer;
unsigned int bufferLen;
{
  unsigned int i;

  for (i = 0; i < bufferLen; ++i) {
    if ( ((i & 7) == 7) || (i == (bufferLen - 1)) )
      printf ("  %02x\n", buffer[i]);
    else
      printf ("  %02x", buffer[i]);
  }

  printf ("\n");
} /*  end PrintBuf  */
